#!/bin/env python3

# MIT License
#
# Copyright (c) 2018-2019 Red Hat, Inc.

# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

from flexmock import flexmock

from container_ci_suite.openshift import OpenShiftAPI


class TestDeployTemplateWithImage(object):
    """
    Tests for OpenShiftAPI.deploy_template_with_image.
    """

    def test_deploy_template_with_image_returns_false_when_project_not_created(self):
        """
        Short-circuits when project_created is False.
        """
        oc_api = OpenShiftAPI(namespace="ns-test")
        oc_api.project_created = False
        assert (
            oc_api.deploy_template_with_image(
                "quay.io/foo",
                "templates/app.json",
                name_in_template="app",
            )
            is False
        )

    def test_deploy_template_with_image_returns_false_when_upload_image_fails(self):
        """
        Non-shared cluster: upload_image failure returns False before deploy_template.
        """
        oc_api = OpenShiftAPI(namespace="ns-test")
        oc_api.project_created = True
        oc_api.shared_cluster = False
        oc_api.version = "14"
        flexmock(OpenShiftAPI).should_receive("upload_image").with_args(
            source_image="quay.io/foo",
            tagged_image="app:14",
        ).and_return(False)
        flexmock(OpenShiftAPI).should_receive("deploy_template").never()

        assert (
            oc_api.deploy_template_with_image(
                "quay.io/foo",
                "templates/app.json",
                name_in_template="app",
            )
            is False
        )

    def test_deploy_template_with_image_returns_false_when_upload_external_fails(self):
        """
        Shared cluster: upload_image_to_external_registry failure returns False.
        """
        oc_api = OpenShiftAPI(namespace="ns-test")
        oc_api.project_created = True
        oc_api.shared_cluster = True
        oc_api.version = ""
        flexmock(OpenShiftAPI).should_receive("upload_image_to_external_registry").with_args(
            source_image="registry.io/img:latest",
            tagged_image="svc:",
        ).and_return(False)
        flexmock(OpenShiftAPI).should_receive("upload_image").never()
        flexmock(OpenShiftAPI).should_receive("deploy_template").never()

        assert (
            oc_api.deploy_template_with_image(
                "registry.io/img:latest",
                "templates/t.json",
                name_in_template="svc",
            )
            is False
        )

    def test_deploy_template_with_image_non_shared_calls_deploy_template(self):
        """
        After a successful upload_image, delegates to deploy_template with expected_output=''.
        """
        oc_api = OpenShiftAPI(namespace="ns-test")
        oc_api.project_created = True
        oc_api.shared_cluster = False
        oc_api.version = "1"
        flexmock(OpenShiftAPI).should_receive("upload_image").with_args(
            source_image="img",
            tagged_image="name:1",
        ).and_return(True)
        flexmock(OpenShiftAPI).should_receive("deploy_template").with_args(
            template="tpl.json",
            name_in_template="name",
            openshift_args=["MEMORY=512Mi"],
            expected_output="",
        ).and_return(True)

        assert (
            oc_api.deploy_template_with_image(
                "img",
                "tpl.json",
                name_in_template="name",
                openshift_args=["MEMORY=512Mi"],
            )
            is True
        )

    def test_deploy_template_with_image_shared_cluster_calls_external_upload(self):
        """
        Shared cluster uses upload_image_to_external_registry with the same tagged name.
        """
        oc_api = OpenShiftAPI(namespace="ns-test")
        oc_api.project_created = True
        oc_api.shared_cluster = True
        oc_api.version = "v2"
        flexmock(OpenShiftAPI).should_receive("upload_image_to_external_registry").with_args(
            source_image="src-img",
            tagged_image="appname:v2",
        ).and_return(True)
        flexmock(OpenShiftAPI).should_receive("upload_image").never()
        flexmock(OpenShiftAPI).should_receive("deploy_template").with_args(
            template="t.json",
            name_in_template="appname",
            openshift_args=None,
            expected_output="",
        ).and_return(True)

        assert (
            oc_api.deploy_template_with_image("src-img", "t.json", name_in_template="appname")
            is True
        )

    def test_deploy_template_with_image_returns_deploy_template_result(self):
        """
        Return value is whatever deploy_template returns.
        """
        oc_api = OpenShiftAPI(namespace="ns-test")
        oc_api.project_created = True
        oc_api.shared_cluster = False
        flexmock(OpenShiftAPI).should_receive("upload_image").and_return(True)
        flexmock(OpenShiftAPI).should_receive("deploy_template").and_return(False)

        assert (
            oc_api.deploy_template_with_image("img", "tpl.json", name_in_template="n")
            is False
        )



#!/bin/env python3

# MIT License
#
# Copyright (c) 2018-2019 Red Hat, Inc.

# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

import os
import yaml

from pathlib import Path
from unittest.mock import patch

from flexmock import flexmock

import pytest

from container_ci_suite.utils import (
    get_public_image_name,
    get_npm_variables,
    get_mount_ca_file,
    get_mount_options_from_s2i_args,
    get_env_commands_from_s2i_args,
    download_template,
)
from container_ci_suite import utils

from tests.conftest import create_ca_file, delete_ca_file


class TestContainerCISuiteUtils(object):
    """
    Test Container CI Suite utils.
    """

    @pytest.mark.parametrize(
        "os,base_image_name,version,expected_str",
        [
            ("rhel8", "nodejs", "14", "registry.redhat.io/rhel8/nodejs-14"),
        ],
    )
    def test_get_public_image_name(self, os, base_image_name, version, expected_str):
        """
        Test get_public_image_name.
        """
        name = get_public_image_name(
            os=os, base_image_name=base_image_name, version=version
        )
        assert name == expected_str

    def test_get_npm_variables_no_ca_file(self):
        """
        Test get_npm_variables_no_ca_file.
        """
        assert get_npm_variables() == ""

    def test_get_mount_ca_file_no_ca_file(self):
        """
        Test get_mount_ca_file_no_ca_file.
        """
        assert get_mount_ca_file() == ""

    def test_get_npm_variables(self):
        """
        Test get_npm_variables.
        """
        create_ca_file()
        flexmock(utils).should_receive("get_full_ca_file_path").and_return(
            Path("/tmp/CA_FILE_PATH")
        )
        assert get_npm_variables() == f"-e NPM_MIRROR=foobar {get_mount_ca_file()}"
        delete_ca_file()

    def test_get_mount_ca_file(self):
        """
        Test get_mount_ca_file.
        """
        create_ca_file()
        assert get_mount_ca_file() == f"{get_mount_ca_file()}"
        delete_ca_file()

    @pytest.mark.parametrize(
        "s2i_args,expected_output",
        [
            ("--pull-never", ""),
            (
                "--pull-never -v /some/foo/bar/file:/some/foo/bar/file:Z",
                "-v /some/foo/bar/file:/some/foo/bar/file:Z",
            ),
            ("-v ./relative/path:/target:Z", "-v ./relative/path:/target:Z"),
            ("-v /host/path:/container:ro", "-v /host/path:/container:ro"),
            ("", ""),
        ],
    )
    def test_get_mount_options_from_s2i_args(self, s2i_args, expected_output):
        """
        Test get_mount_options_from_s2i_args.
        """
        ret_value = get_mount_options_from_s2i_args(s2i_args=s2i_args)
        assert ret_value == expected_output

    @pytest.mark.parametrize(
        "s2i_args,expected_output",
        [
            ("--pull-never", []),
            ("--pull-never -e NODE=development", ["ENV NODE=development"]),
            ("--pull-never -e=NODE=development", ["ENV NODE=development"]),
            (
                "-v mount_point:mount_point:Z -e FOO=bar --env TEST=deployment",
                ["ENV FOO=bar", "ENV TEST=deployment"],
            ),
            ("-v mount_point:mount_point:Z -e FOO=bar --env TEST", ["ENV FOO=bar"]),
            ("-v mount_point:mount_point:Z -e=FOO=bar --env=TEST", ["ENV FOO=bar"]),
            ("", []),
            ("-e KEY1=val1 -e KEY2=val2", ["ENV KEY1=val1", "ENV KEY2=val2"]),
        ],
    )
    def test_get_env_commands_from_s2i_args(self, s2i_args, expected_output):
        """
        Test get_env_commands_from_s2i_args.
        """
        assert get_env_commands_from_s2i_args(s2i_args=s2i_args) == expected_output

    @pytest.mark.parametrize(
        "image_name,version,os_name,expected_output",
        [
            ("rhel8/ubi8", "", "", False),
            ("", "2.4", "", False),
            ("", "", "rhel8", False),
            ("rhel8/httpd-24:1", "2.4", "rhel8", True),
            ("rhel8/httpd-24", "2.4", "rhel9", True),
            ("registry.io/ns/image:1.0", "2.0", "rhel8", True),
        ],
    )
    def test_check_variables(self, image_name, version, os_name, expected_output):
        """
        Test check_variables.
        """
        os.environ["IMAGE_NAME"] = image_name
        os.environ["VERSION"] = version
        os.environ["OS"] = os_name
        assert utils.check_variables() == expected_output

    @pytest.mark.parametrize(
        "image_name,version,expected_output",
        [
            ("rhel8/httpd-24:1", "2.4", "httpd-24:2.4"),
            ("/httpd-24:1", "2.4", "httpd-24:2.4"),
            ("", "2.4", None),
            ("rhel8/httpd", "2.4", "httpd:2.4"),
            ("registry.io/namespace/image:1.0", "2.0", "namespace:2.0"),
            ("singlepart", "1.0", None),
            ("registry.io/ubi8:8", "2.0", "ubi8:2.0"),
        ],
    )
    def test_get_tagged_image(self, image_name, version, expected_output):
        """
        Test get_tagged_image.
        """
        assert (
            utils.get_tagged_image(image_name=image_name, version=version)
            == expected_output
        )

    @pytest.mark.parametrize(
        "image_name,expected_output",
        [
            ("rhel8/httpd-24:1", "httpd-24-testing"),
            ("/httpd-24:1", "httpd-24-testing"),
            ("", None),
            ("rhel8/httpd", "httpd-testing"),
        ],
    )
    def test_get_service_image(self, image_name, expected_output):
        """
        Test get_service_image.
        """
        assert utils.get_service_image(image_name=image_name) == expected_output

    def test_get_image_name_no_image_id_file(self, temp_dir):
        """
        Test get_image_name when .image-id file does not exist.
        """
        assert utils.get_image_name(str(temp_dir)) is None

    @patch("container_ci_suite.utils.ContainerTestLibUtils.run_command")
    @patch("container_ci_suite.utils.get_file_content")
    def test_get_image_name_success(
        self, mock_get_file_content, mock_run_command, temp_dir
    ):
        """
        Test get_image_name when .image-id exists and docker inspect succeeds.
        """
        image_id_file = temp_dir / ".image-id"
        image_id_file.write_text("sha256:abc123\n")
        mock_get_file_content.return_value = "sha256:abc123"
        mock_run_command.side_effect = ["httpd-24", "2.4"]
        result = utils.get_image_name(str(temp_dir))
        assert result == "httpd-24:2.4"
        assert mock_run_command.call_count == 2

    def test_tenantnamespace_yaml(self):
        """
        Test tenantnamespace_yaml.
        """
        expected_yaml = {
            "apiVersion": "tenant.paas.redhat.com/v1alpha1",
            "kind": "TenantNamespace",
            "metadata": {
                "labels": {
                    "tenant.paas.redhat.com/namespace-type": "build",
                    "tenant.paas.redhat.com/tenant": "core-services-ocp",
                },
                "name": "123456",
                "namespace": "core-services-ocp--config",
            },
            "spec": {
                "type": "build",
                "roles": ["namespace-admin", "tenant-egress-admin"],
                "network": {"security-zone": "internal"},
            },
        }
        tenant_yaml = utils.save_tenant_namespace_yaml(project_name="123456")
        with open(tenant_yaml) as fd:
            yaml_load = yaml.safe_load(fd.read())
        assert yaml_load
        assert yaml_load["metadata"]["name"] == "123456"
        assert yaml_load == expected_yaml

    def test_tenantnegress_yaml(self):
        """
        Test tenantnegress_yaml.
        """
        tenant_yaml = utils.save_tenant_egress_yaml(project_name="123456")
        with open(tenant_yaml) as fd:
            yaml_load = yaml.safe_load(fd.read())
        assert yaml_load
        assert yaml_load["metadata"]["namespace"] == "core-services-ocp--123456"
        assert yaml_load["spec"]["egress"][0]["to"]["dnsName"] == "github.com"
        check_address = False
        check_dnsName = False
        check_deny = False
        for egress in yaml_load["spec"]["egress"]:
            if "cidrSelector" in egress["to"]:
                if egress["to"]["cidrSelector"] == "52.92.128.0/17":
                    check_address = True
                if (
                    egress["to"]["cidrSelector"] == "0.0.0.0/0"
                    and egress["type"] == "Deny"
                ):
                    check_deny = True
            if "dnsName" in egress["to"]:
                if egress["to"]["dnsName"] == "registry.npmjs.org":
                    check_dnsName = True
        assert check_address
        assert check_dnsName
        assert check_deny

    def test_tenantlimits_yaml(self):
        """
        Test tenantlimits_yaml.
        """
        tenant_limits_yaml = utils.save_tenant_limit_yaml()
        with open(tenant_limits_yaml) as fd:
            yaml_load = yaml.safe_load(fd.read())
        assert yaml_load
        assert len(yaml_load["spec"]["limits"]) == 2
        check_pod_max = False
        check_pod_min = False
        check_container_max = False
        check_container_min = False
        for limits in yaml_load["spec"]["limits"]:
            if "Pod" in limits["type"]:
                if limits["max"]["cpu"] == "8" and limits["max"]["memory"] == "8Gi":
                    check_pod_max = True
                if limits["min"]["cpu"] == "4" and limits["min"]["memory"] == "2Gi":
                    check_pod_min = True
            if "Container" in limits["type"]:
                if limits["max"]["cpu"] == "8" and limits["max"]["memory"] == "8Gi":
                    check_container_max = True
                if limits["min"]["cpu"] == "2" and limits["min"]["memory"] == "2Gi":
                    check_container_min = True
        assert check_pod_max
        assert check_pod_min
        assert check_container_max
        assert check_container_min

    def test_download_template_local_file(self, temp_dir):
        """
        Test download_template with a local file.
        """
        source_file = temp_dir / "template.yaml"
        source_file.write_text("apiVersion: v1\nkind: Template")
        result = download_template(str(source_file), dir_name=str(temp_dir))
        assert result is not None
        assert Path(result).exists()
        assert Path(result).suffix == ".yaml"
        assert Path(result).read_text() == "apiVersion: v1\nkind: Template"

    def test_download_template_local_directory(self, temp_dir):
        """
        Test download_template with a local directory.
        """
        source_dir = temp_dir / "template_dir"
        source_dir.mkdir()
        (source_dir / "file.txt").write_text("content")
        result = download_template(str(source_dir), dir_name=str(temp_dir))
        assert result is not None
        assert Path(result).is_dir()
        assert (Path(result) / "file.txt").read_text() == "content"

    def test_download_template_nonexistent(self, temp_dir):
        """
        Test download_template with non-existent path.
        """
        result = download_template("/nonexistent/path/12345", dir_name=str(temp_dir))
        assert result is None

    @patch("container_ci_suite.utils.requests.get")
    def test_download_template_http_url_fails(self, mock_get, temp_dir):
        """
        Test download_template when HTTP request fails.
        """
        import requests

        mock_get.side_effect = requests.HTTPError("404 Not Found")
        with pytest.raises(requests.HTTPError):
            download_template(
                "https://example.com/missing.yaml", dir_name=str(temp_dir)
            )

    @pytest.mark.parametrize(
        "json_data,expected_output",
        [
            ({"helm": "False"}, False),
            ({"helm": "True"}, True),
            ({"helm": True}, True),
            ({"ocp4": "False"}, False),
            ({"ocp4": "True"}, True),
            ({"ocp4": True}, True),
            ({"ocp4": "false"}, False),
            ({"ocp4": None}, False),
            ({"ocp4": "true"}, True),
            ({"ocp4": ""}, False),
            ({"ocp4": "Somthing"}, False),
            ({"ocp4": "Y"}, True),
            ({"helm": "y"}, True),
            ({"helm": "1"}, True),
            ({"helm": "No"}, False),
            ({"helm": "0"}, False),
            ({"helm": "true"}, True),
        ],
    )
    def test_is_shared_cluster(self, json_data, expected_output):
        """
        Test is_shared_cluster.
        """
        flexmock(utils).should_receive("get_json_data").and_return(json_data)
        test = json_data.keys()
        assert utils.is_shared_cluster(test_type="".join(test)) == expected_output

    def test_is_shared_cluster_key_not_present(self):
        """
        Test is_shared_cluster when test_type key is not in json data.
        """
        flexmock(utils).should_receive("get_json_data").and_return({"helm": True})
        assert utils.is_shared_cluster(test_type="ocp4") is False



# MIT License
#
# Copyright (c) 2018-2019 Red Hat, Inc.

# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
import json
import logging
import re
import shlex
import time
import random
import subprocess


from pathlib import Path
from time import sleep
from typing import Dict, Any, List

from container_ci_suite.engines.container import PodmanCLIWrapper
from container_ci_suite.engines.openshift import OpenShiftOperations
from container_ci_suite.utils import ContainerTestLibUtils


import container_ci_suite.utils as utils

logging.basicConfig(format="%(levelname)s:%(message)s", level=logging.DEBUG)
logger = logging.getLogger(__name__)


class OpenShiftAPI:
    """
    OpenShift API - Utility functions for OpenShift operations.
    """

    def __init__(
        self,
        namespace: str = "default",
        pod_name_prefix: str = "",
        create_prj: bool = True,
        delete_prj: bool = True,
        shared_cluster: bool = False,
        version: str = "",
        test_type: str = "ocp4",
    ):
        self.create_prj = create_prj
        self.delete_prj = delete_prj
        if not shared_cluster:
            self.shared_cluster = shared_cluster
        else:
            self.shared_cluster = utils.is_shared_cluster(test_type=test_type)
        self.pod_name_prefix = pod_name_prefix
        self.pod_json_data: Dict = {}
        self.version = version
        self.shared_random_name = ""
        self.config_tenant_name = "core-services-ocp--config"
        self.openshift_ops = OpenShiftOperations(pod_name_prefix=pod_name_prefix)
        self.project_created: bool = False
        if namespace == "default":
            self.create_project()
        else:
            self.namespace = namespace
            self.create_prj = False
        logger.debug(
            "Namespace is: %s and shared cluster is: %s", namespace, self.shared_cluster
        )

    def _create_openshift_project(self) -> bool:
        """
        Create an OpenShift project.
        Returns:
            True if the project was created, False otherwise
        """
        logger.debug("Trying to create project with the name '%s'", self.namespace)
        try:
            ContainerTestLibUtils.run_oc_command(
                f"new-project {self.namespace}",
                json_output=False,
                return_output=True,
            )
            return True
        except subprocess.CalledProcessError:
            logger.error(
                "Project with the name '%s' were not created.",
                self.namespace,
                exc_info=True,
            )
            return False

    def create_project(self):
        """
        Create a project.
        """
        logger.debug("Create project %s and %s", self.create_prj, self.shared_cluster)
        if self.create_prj:
            self.openshift_ops.login_to_cluster(shared_cluster=self.shared_cluster)
            if self.shared_cluster:
                self.shared_random_name = f"sclorg-{random.randrange(10000, 100000)}"
                self.namespace = f"core-services-ocp--{self.shared_random_name}"
                self.openshift_ops.set_namespace(self.namespace)
                if not self.prepare_tenant_namespace():
                    return False
            else:
                for _ in range(3):
                    self.namespace = f"sclorg-{random.randrange(10000, 100000)}"
                    self.openshift_ops.set_namespace(self.namespace)
                    if not self._create_openshift_project():
                        sleep(3)
                        continue
                    break
                else:
                    logger.error(
                        "Project with the name '%s' were not created after 3 attempts.",
                        self.namespace,
                    )
                    return False
                logger.debug("Project with the name '%s' were created.", self.namespace)
        else:
            ContainerTestLibUtils.run_oc_command(
                f"project {self.namespace}", json_output=False
            )
        self.project_created = True
        return self.openshift_ops.is_project_exists()

    def create_tenant_namespace(self) -> bool:
        """
        Create a tenant namespace.
        Returns:
            True if the tenant namespace was created, False otherwise
        """
        tenant_yaml_file = utils.save_tenant_namespace_yaml(
            project_name=self.shared_random_name
        )
        tentant_output = ContainerTestLibUtils.run_oc_command(
            cmd=f"create -f {tenant_yaml_file}",
            json_output=False,
            ignore_error=True,
            return_output=False,
            debug=True,
        )
        if tentant_output != 0:
            logger.error(
                "Create tenant namespace with the name '%s' was not successful.%s",
                self.shared_random_name,
                tentant_output,
            )
            return False
        return True

    def is_tenant_namespace_created(self) -> bool:
        """
        Check if the tenant namespace was created.
        Returns:
            True if the tenant namespace was created, False otherwise
        """
        is_tenant_namespace_created = False
        logger.debug(
            "Check if TenantNamespace %s really exists", self.shared_random_name
        )
        for _ in range(5):
            logger.debug("Waiting for TenantNamespace creation...")
            if not self.openshift_ops.is_project_exists():
                sleep(5)
                continue
            is_tenant_namespace_created = True
            break
        if not is_tenant_namespace_created:
            logger.error(
                "\nTenantNamespace %s was not created properly.",
                self.shared_random_name,
            )
        return is_tenant_namespace_created

    def apply_tenant_egress_rules(self) -> bool:
        """
        Apply the tenant egress rules.
        Returns:
            True if the tenant egress rules were applied, False otherwise
        """
        tenant_egress_file = utils.save_tenant_egress_yaml(
            project_name=self.shared_random_name
        )
        is_applied = False
        for _ in range(10):
            tentant_output = ContainerTestLibUtils.run_oc_command(
                cmd=f"apply -f {tenant_egress_file}",
                json_output=False,
                ignore_error=True,
                return_output=False,
                debug=True,
            )
            if tentant_output != 0:
                logger.error(
                    "Apply egress rules to tenant namespace '%s' was not successful.%s. Let's try one more tine",
                    self.shared_random_name,
                    tentant_output,
                )
                sleep(5)
                continue
            is_applied = True
            break
        return is_applied

    def create_limit_ranges(self) -> bool:
        """
        Create the limit ranges.
        Returns:
            True if the limit ranges were created, False otherwise
        """
        tenant_limit_file = utils.save_tenant_limit_yaml()
        is_applied = False
        for _ in range(10):
            tentant_output = ContainerTestLibUtils.run_oc_command(
                cmd=f"apply -f {tenant_limit_file}",
                json_output=False,
                ignore_error=True,
                return_output=False,
                debug=True,
            )
            if tentant_output != 0:
                logger.error(
                    "create limit ranges to tenant namespace '%s' was not successful.%s. Let's try one more tine",
                    self.shared_random_name,
                    tentant_output,
                )
                sleep(5)
                continue
            is_applied = True
            break
        return is_applied

    def prepare_tenant_namespace(self):
        """
        Prepare the tenant namespace.
        """
        logger.info("Prepare Tenant Namespace with name: '%s'", self.shared_random_name)
        json_flag = False
        try:
            namespace = ContainerTestLibUtils.run_oc_command(
                cmd="project -q", json_output=json_flag
            ).strip()
            logger.debug("The current namespace is '%s'", namespace)
            if namespace != self.config_tenant_name:
                ContainerTestLibUtils.run_oc_command(
                    f"project {self.config_tenant_name}", json_output=json_flag
                )
        except subprocess.CalledProcessError:
            ContainerTestLibUtils.run_oc_command(
                f"project {self.config_tenant_name}", json_output=json_flag
            )
        if not self.create_tenant_namespace():
            return False
        # Let's wait 5 seconds till project is not up
        time.sleep(10)
        if not self.is_tenant_namespace_created():
            return False
        if not self.apply_tenant_egress_rules():
            return False
        self.project_created = True
        ContainerTestLibUtils.run_oc_command(
            cmd=f"project {self.namespace}", json_output=json_flag, return_output=True
        )
        # if not self.create_limit_ranges():
        #     return False
        logger.info("Tenant Namespace were created")

    def delete_tenant_namespace(self):
        """
        Delete the tenant namespace.
        """
        json_flag = False
        namespace = ContainerTestLibUtils.run_oc_command(
            cmd="project -q", json_output=json_flag
        )
        if namespace == self.config_tenant_name:
            logger.error(
                "Deleting tenant '%s' is not allowed.", self.config_tenant_name
            )
            return
        ContainerTestLibUtils.run_oc_command(
            f"project {self.config_tenant_name}", json_output=json_flag
        )
        if ContainerTestLibUtils.run_oc_command(
            f"delete tenantnamespace {self.shared_random_name}", json_output=json_flag
        ):
            logger.info(
                "TenantNamespace %s was deleted properly",
                self.shared_random_name,
            )
        else:
            logger.error(
                "!!!!! TenantNamespace %s was not delete properly. But it does not block CI.!!!!",
                self.shared_random_name,
            )

    def get_raw_url_for_json(
        self, container: str, dir: str, filename: str, branch: str = "master"
    ):
        """
        Get the raw URL for the JSON.
        Args:
            container: The container
            dir: The directory
            filename: The filename
            branch: The branch
        Returns:
            The raw URL for the JSON
        """
        return utils.get_raw_url_for_json(
            container=container, dir=dir, filename=filename, branch=branch
        )

    def is_s2i_pod_running(self, pod_name_prefix: str, cycle_count: int = 180):
        """
        Check if the S2I pod is running.
        Args:
            pod_name_prefix: The prefix for the pod name
            cycle_count: The number of loops to check
        Returns:
            True if the S2I pod is running, False otherwise
        """
        return self.openshift_ops.is_s2i_pod_running(
            pod_name_prefix=pod_name_prefix, cycle_count=cycle_count
        )

    def is_pod_running(self, pod_name_prefix: str):
        """
        Check if the pod is running.
        Args:
            pod_name_prefix: The prefix for the pod name
        Returns:
            True if the pod is running, False otherwise
        """
        return self.openshift_ops.is_pod_running(pod_name_prefix=pod_name_prefix)

    def delete_project(self):
        """
        Delete the project.
        """
        if not self.delete_prj:
            logger.info("Deleting project is SUPPRESSED.")
            # project is not deleted by request user
        else:
            if self.shared_cluster:
                logger.info("Delete project on shared cluster")
                self.delete_tenant_namespace()
            else:
                logger.info("Deleting project %s", self.namespace)
                ContainerTestLibUtils.run_oc_command(
                    "project default", json_output=False
                )
                ContainerTestLibUtils.run_oc_command(
                    f"delete project {self.namespace} --grace-period=0 --force",
                    json_output=False,
                )

    def run_command_in_pod(self, pod_name, command: str = "") -> str:
        """
        Run a command in a pod.
        Args:
            pod_name: The name of the pod
            command: The command to run
        Returns:
            The output of the command
        """
        output = ContainerTestLibUtils.run_oc_command(f'exec {pod_name} -- "{command}"')
        logger.debug("%s", output)
        return output

    def import_is(self, path: str, name: str, skip_check=False):
        """
        Import an image stream.
        Args:
            path: The path to the image stream
            name: The name of the image stream
            skip_check: Whether to skip the check
        Returns:
            The output of the command
        """
        if not skip_check:
            is_exists = self.openshift_ops.is_imagestream_exist(name=name)
            if is_exists:
                return is_exists
        output = ContainerTestLibUtils.run_oc_command(
            f"create -f {path}", namespace=self.namespace
        )
        # Let's wait 3 seconds till imagestreams are not uploaded
        time.sleep(3)
        return json.loads(output)

    def process_file(self, path: str):
        """
        Process a file.
        Args:
            path: The path to the file
        Returns:
            The output of the command
        """
        output = ContainerTestLibUtils.run_oc_command(
            f"process -f {path}", namespace=self.namespace
        )
        json_output = json.loads(output)
        logger.debug(json_output)
        return json_output

    def start_build(self, service_name: str, app_name: str = "") -> str:
        """
        Start a build.
        Args:
            service_name: The name of the service
            app_name: The name of the app
        Returns:
            The output of the command
        """
        from_dir = utils.download_template(template_name=app_name)
        output = ContainerTestLibUtils.run_oc_command(
            f"start-build {service_name} --from-dir={from_dir}", json_output=False
        )
        return output

    def login_to_shared_cluster(self):
        """
        Login to the shared cluster.
        """
        self.openshift_ops.login_to_cluster(shared_cluster=True)
        output = ContainerTestLibUtils.run_oc_command("version", json_output=False)
        logger.debug(output)
        output = ContainerTestLibUtils.run_oc_command(
            f"project {self.config_tenant_name}", json_output=False
        )
        logger.debug("%s", output)

    @staticmethod
    def login_external_registry() -> Any:
        """
        Login to the external registry.
        Returns:
            The registry URL
        """
        registry_url = utils.get_shared_variable("registry_url")
        robot_token = utils.load_shared_credentials("ROBOT_TOKEN")
        robot_name = utils.get_shared_variable("robot_account")
        if not all([registry_url, robot_token, robot_name]):
            logger.error(
                "Important variable ROBOT_TOKEN or variables in file /root/shared_cluster"
                " 'registry_url,robot_account' are missing."
            )
            return None
        cmd = f'podman login -u "{robot_name}" -p "{robot_token}" {registry_url}'
        output = ContainerTestLibUtils.run_command(
            cmd=cmd, ignore_error=False, return_output=True
        )
        logger.debug("Output from podman login: %s", output)
        return registry_url

    def upload_image_to_external_registry(self, source_image: str, tagged_image: str):
        """
        Upload an image to the external registry.
        Args:
            source_image: The source image
            tagged_image: The tagged image
        Returns:
            The output of the command
        """
        register_url = OpenShiftAPI.login_external_registry()
        logger.debug("Registry_url: %s", register_url)
        if not register_url:
            return None
        output_name = f"{register_url}/core-services-ocp/{tagged_image}"
        logger.debug("%s", ContainerTestLibUtils.run_command("podman images"))
        cmd = f"podman tag {source_image} {output_name}"
        output = ContainerTestLibUtils.run_command(
            cmd, ignore_error=False, return_output=True
        )
        logger.debug("Output from podman tag command %s", output)
        cmd = f"podman push {output_name}"
        output = ContainerTestLibUtils.run_command(
            cmd, ignore_error=False, return_output=True
        )
        logger.debug("Output from podman push command %s", output)
        ret = ContainerTestLibUtils.run_oc_command(
            f"import-image {tagged_image} --from={output_name} --confirm",
            json_output=False,
            return_output=True,
        )
        logger.debug("%s", ret)
        # Let's wait couple seconds
        time.sleep(3)
        return True

    def podman_login_to_openshift(self) -> Any:
        """
        Login to the OpenShift.
        Returns:
            The OpenShift URL
        """
        output = ContainerTestLibUtils.run_oc_command(
            cmd="get route default-route -n openshift-image-registry"
        )
        jsou_output = json.loads(output)
        logger.debug(jsou_output["spec"]["host"])
        if not jsou_output["spec"]["host"]:
            logger.error(
                "Default route does not exist. Install OpenShift 4 cluster properly and expose default route."
            )
            return None
        ocp4_register = jsou_output["spec"]["host"]
        token_output = ContainerTestLibUtils.run_oc_command(
            cmd="whoami -t", json_output=False
        ).strip()
        cmd = f"podman login -u kubeadmin -p {token_output} {ocp4_register}"
        output = ContainerTestLibUtils.run_command(
            cmd=cmd, ignore_error=False, return_output=True
        )
        logger.debug("Output from podman login: %s", output)
        return ocp4_register

    def upload_image(self, source_image: str, tagged_image: str) -> bool:
        """
        Function pull the image specified by parameter source_image
        and tagged it into OpenShift environment as tagged_image
        :param source_image: image that is pulled and uploaded to OpenShift
        :param tagged_image: image uploaded to OpenShift and tagged by this parameter
        :return True: image was properly uploaded to OpenShift
                False: image was not either pulled or uploading failed
        """
        if not PodmanCLIWrapper.podman_pull_image(image_name=source_image, loops=3):
            return False
        try:
            ocp4_register = self.podman_login_to_openshift()
            if not ocp4_register:
                return False
        except subprocess.CalledProcessError:
            return False
        output_name = f"{ocp4_register}/{self.namespace}/{tagged_image}"
        cmd = f"podman tag {source_image} {output_name}"
        logger.debug("Tag podman image %s", cmd)
        output = ContainerTestLibUtils.run_command(cmd=cmd, ignore_error=False)
        logger.debug("Upload_image tagged %s", output)
        output = ContainerTestLibUtils.run_command(
            cmd=f"podman push {output_name}", ignore_error=False
        )
        logger.debug("Upload_image push %s", output)
        return True

    def create_new_app_with_template(
        self, name: str, template_json: str, template_args: Dict = None
    ):
        """
        Function creates a new application in OpenShift 4 environment
        :param name: str - Template name
        :param template_json: str - Template path to file
        :param template_args: Dict - Arguments that will be passed to oc new-app
        :return json toutput
        """
        if not self.project_created:
            return False
        # Let's wait couple seconds till is fully loaded
        time.sleep(3)
        args = [""]
        if template_args:
            args = [f"-p {key}={val}" for key, val in template_args]
        output = ContainerTestLibUtils.run_oc_command(
            f"new-app {template_json} --name {name} -p NAMESPACE={self.namespace} {args}",
            json_output=False,
        )
        logger.debug("%s", output)
        return output

    def get_route_url(self, routes_name: str) -> Any:
        """
        Get the route URL.
        Args:
            routes_name: The name of the routes
        Returns:
            The route URL
        """
        output = ContainerTestLibUtils.run_oc_command(f"get routes/{routes_name}")
        json_output = json.loads(output)
        if not json_output["spec"]["host"]:
            return None
        if routes_name != json_output["spec"]["to"]["name"]:
            return None
        return json_output["spec"]["host"]

    def get_openshift_args(self, oc_args: List[str]) -> str:
        """
        Get the OpenShift arguments.
        Args:
            oc_args: The OpenShift arguments
        Returns:
            The OpenShift arguments
        """
        return " -p ".join(oc_args)

    def is_template_deployed(
        self, name_in_template: str = "", timeout: int = 180
    ) -> bool:
        """
        Check if the template was deployed properly.
        Args:
            name_in_template: The name of the template
            timeout: The timeout
        Returns:
            True if the template was deployed properly, False otherwise
        """
        logger.info("Check if template was deployed properly")
        if not self.openshift_ops.is_build_pod_finished(cycle_count=timeout):
            logger.error("\nBuild pod does not finished in proper time")
            self.openshift_ops.print_get_status()
            return False
        if not self.openshift_ops.is_pod_running(pod_name_prefix=name_in_template):
            logger.error("Pod is not running after time.")
            self.openshift_ops.print_get_status()
            return False
        return True

    def command_app_run(self, cmd: str, return_output: bool = True) -> str:
        """
        Run a command in the command app.
        Args:
            cmd: The command to run
            return_output: Whether to return the output
        Returns:
            The output of the command
        """
        safe_cmd = shlex.quote(cmd)
        cmd = f"exec command-app -- bash -c {safe_cmd}"
        logger.debug("command_app_run: %s", cmd)
        cmd_out = ContainerTestLibUtils.run_oc_command(
            cmd=cmd, ignore_error=True, return_output=return_output, json_output=False
        )
        return cmd_out

    def create_deploy_command_app(
        self, image_name: str = "registry.access.redhat.com/ubi8/ubi"
    ) -> bool:
        """
        Create a deploy command app.
        Args:
            image_name: The name of the image
        Returns:
            True if the deploy command app was created, False otherwise
        """
        cmd_file = utils.save_command_yaml(image_name=image_name)
        ContainerTestLibUtils.run_oc_command(f"create -f {cmd_file}")
        if not self.openshift_ops.is_pod_running(pod_name_prefix="command-app"):
            logger.error(
                "create_deploy_command_app: command-app pod is not running after time."
            )
            self.openshift_ops.print_get_status()
            return False
        output_cmd = self.command_app_run("echo $((11*11))")
        if "121" not in output_cmd:
            return False
        return True

    def imagestream_quickstart(
        self,
        imagestream_file: str,
        template_file: str,
        image_name: str,
        name_in_template: str,
        openshift_args=None,
    ) -> bool:
        """
        Deploy an imagestream quickstart.
        Args:
            imagestream_file: The path to the imagestream file
            template_file: The path to the template file
            image_name: The name of the image
            name_in_template: The name of the template
            openshift_args: The OpenShift arguments
        Returns:
            True if the imagestream quickstart was deployed, False otherwise
        """
        if not self.project_created:
            return False
        local_imagestream_file = utils.download_template(imagestream_file)
        self.import_is(local_imagestream_file, name="", skip_check=True)
        tagged_image = utils.get_tagged_image(
            image_name=image_name, version=self.version
        )
        if self.shared_cluster:
            if not self.upload_image_to_external_registry(
                source_image=image_name, tagged_image=tagged_image
            ):
                return False
        else:
            if not self.upload_image(
                source_image=image_name, tagged_image=tagged_image
            ):
                return False
        return self.deploy_template_with_image(
            image_name=image_name,
            template=template_file,
            name_in_template=name_in_template,
            openshift_args=openshift_args,
        )

    def deploy_s2i_app(
        self, image_name: str, app: str, context: str, service_name: str = ""
    ) -> bool:
        """
        Deploy an S2I app.
        Args:
            image_name: The name of the image
            app: The path to the app
            context: The context of the app
            service_name: The name of the service
        Returns:
            True if the S2I app was deployed, False otherwise
        """
        if not self.project_created:
            return False
        tagged_image = utils.get_tagged_image(
            image_name=image_name, version=self.version
        )
        logger.info("Source image %s was tagged as %s", image_name, tagged_image)
        if self.shared_cluster:
            if not self.upload_image_to_external_registry(
                source_image=image_name, tagged_image=tagged_image
            ):
                return False
        else:
            if not self.upload_image(
                source_image=image_name, tagged_image=tagged_image
            ):
                return False

        if service_name == "":
            service_name = utils.get_service_image(image_name)
        logger.info("Service name in app is: %s", service_name)
        app_param = app
        if Path(app).is_dir():
            app_param = utils.download_template(template_name=app)
        oc_cmd = f"new-app {tagged_image}~{app_param} --strategy=source --context-dir={context} --name={service_name}"
        logger.debug("Command for deploying application is: %s", oc_cmd)
        try:
            output = ContainerTestLibUtils.run_oc_command(
                f"{oc_cmd}", json_output=False
            )
            logger.debug("%s", output)
        except subprocess.CalledProcessError as cpe:
            logger.error(cpe.output)
            return False

        time.sleep(3)
        if Path(app).is_dir():
            output = self.start_build(service_name=service_name, app_name=app)
            logger.info("Output from start build: %s", output)

        return True

    def update_template_example_file(self, file_name: str) -> dict:
        """
        Update the template example file.
        Args:
            file_name: The name of the file
        Returns:
            The JSON data
        """
        json_data = utils.get_json_data(file_name=Path(file_name))
        if "objects" not in json_data:
            return json_data
        for json_object in json_data["objects"]:
            if json_object["kind"] != "PersistentVolumeClaim":
                continue
            if "annotations" not in json_object["metadata"]:
                annotations: Dict = {}
            else:
                annotations = json_object["metadata"]["annotations"]
            annotations["trident.netapp.io/reclaimPolicy"] = "Delete"
            json_object["metadata"]["annotations"] = annotations
            if "labels" not in json_object["metadata"]:
                labels: Dict = {}
            else:
                labels = json_object["metadata"]["labels"]
            labels["paas.redhat.com/appcode"] = utils.get_shared_variable("app_code")
            json_object["metadata"]["labels"] = labels
            json_object["spec"]["storageClassName"] = "netapp-nfs"
            json_object["spec"]["volumeMode"] = "Filesystem"
        utils.dump_json_data(json_data=json_data, file_name=Path(file_name))
        return json_data

    def deploy_image_stream_template(
        self,
        imagestream_file: str,
        template_file: str,
        app_name: str,
        openshift_args=None,
    ) -> bool:
        """
        Deploy an image stream template.
        Args:
            imagestream_file: The path to the imagestream file
            template_file: The path to the template file
            app_name: The name of the app
            openshift_args: The OpenShift arguments
        Returns:
            True if the image stream template was deployed, False otherwise
        """
        if not self.project_created:
            return False
        local_is_file = utils.download_template(template_name=imagestream_file)
        local_template = utils.download_template(template_name=template_file)
        if self.shared_cluster:
            self.update_template_example_file(file_name=local_template)
        json_output = self.import_is(local_is_file, name="", skip_check=True)
        if not json_output:
            logger.error("deploy_image_stream_template: import_is failed")
            return False
        if openshift_args is None:
            openshift_args = ""
        else:
            openshift_args = f"-p {self.get_openshift_args(oc_args=openshift_args)}"
        logger.info(
            "========\nCreating a new-app with name %s in namespace %s with args %s\n========",
            app_name,
            self.namespace,
            openshift_args,
        )
        oc_cmd = f"new-app -f {local_template} --name={app_name} -p NAMESPACE={self.namespace} {openshift_args}"
        logger.info("Deploy template by command: oc %s", oc_cmd)
        try:
            output = ContainerTestLibUtils.run_oc_command(
                f"{oc_cmd}", json_output=False
            )
            logger.info("%s", output)
        except subprocess.CalledProcessError:
            return False
        # Let's wait couple seconds to deployment can start
        time.sleep(3)
        return True

    def deploy_imagestream_s2i(
        self,
        imagestream_file: str,
        image_name: str,
        app: str,
        context: str,
        service_name: str,
    ) -> bool:
        """
        Function deploys imagestreams as s2i application
        In case of failure check if imagestream_file really exist
        :param imagestream_file: imagestream file that is imported to OCP4
        :param image_name: image name that is used for testing
        :param app: the app reference that is used in template like https://github.com/sclorg/httpd-ex.git
        :param context: specify context of in source git repository
        :param service_name: specify service name to check in pods
        :return True: application was properly deployed
                False: application was not properly deployed
        """
        if not self.project_created:
            return False
        imagestream_file = re.sub(r"[0-9]", "", imagestream_file)
        local_template = utils.download_template(template_name=imagestream_file)
        if not local_template:
            return False
        if self.shared_cluster:
            self.update_template_example_file(file_name=local_template)

        self.import_is(local_template, name="", skip_check=True)
        return self.deploy_s2i_app(
            image_name=image_name, app=app, context=context, service_name=service_name
        )

    def deploy_template_with_image(
        self,
        image_name: str,
        template: str,
        name_in_template: str = "",
        openshift_args=None,
    ) -> bool:
        """
        Deploy a template with an image.
        Args:
            image_name: The name of the image
            template: The path to the template
            name_in_template: The name of the template
            openshift_args: The OpenShift arguments
        Returns:
            True if the template was deployed, False otherwise
        """
        if not self.project_created:
            return False
        tagged_image = f"{name_in_template}:{self.version}"
        logger.debug(
            "deploy_template_with_image: %s and %s", tagged_image, self.shared_cluster
        )
        if self.shared_cluster:
            if not self.upload_image_to_external_registry(
                source_image=image_name, tagged_image=tagged_image
            ):
                return False
        else:
            if not self.upload_image(
                source_image=image_name, tagged_image=tagged_image
            ):
                return False

        return self.deploy_template(
            template=template,
            name_in_template=name_in_template,
            openshift_args=openshift_args,
            expected_output="",
        )

    def deploy_template(
        self,
        template: str,
        name_in_template: str,
        expected_output: str,
        port: int = 8080,
        protocol: str = "http",
        response_code: int = 200,
        openshift_args=None,
        other_images=None,
    ) -> bool:
        """
        Deploy a template.
        Args:
            template: The path to the template
            name_in_template: The name of the template
            expected_output: The expected output
            port: The port
            protocol: The protocol
            response_code: The response code
            openshift_args: The OpenShift arguments
            other_images: The other images
        Returns:
            True if the template was deployed, False otherwise
        """
        if not self.project_created:
            return False
        if other_images is None:
            other_images = ""
        if openshift_args is None:
            openshift_args = ""
        else:
            openshift_args = f"-p {self.get_openshift_args(openshift_args)}"
        logger.info(
            "========\nCreating a new-app with name %s in namespace %s with args %s\n========",
            name_in_template,
            self.namespace,
            openshift_args,
        )
        local_template = utils.download_template(template_name=template)
        if not local_template:
            return False
        if self.shared_cluster:
            self.update_template_example_file(file_name=local_template)
        oc_cmd = f"new-app {local_template} --name {name_in_template} -p NAMESPACE={self.namespace} {openshift_args}"
        logger.info("Deploy template by command: oc %s", oc_cmd)
        try:
            output = ContainerTestLibUtils.run_oc_command(
                f"{oc_cmd}", json_output=False
            )
            logger.info("%s", output)
        except subprocess.CalledProcessError:
            return False
        # Let's wait couple seconds to deployment can start
        time.sleep(3)
        return True

    def check_command_internal(
        self,
        image_name: str,
        service_name: str,
        cmd: str,
        expected_output: str,
        timeout: int = 120,
    ) -> bool:
        """
        Check the command internal.
        Args:
            image_name: The name of the image
            service_name: The name of the service
            cmd: The command to run
            expected_output: The expected output
            timeout: The timeout
        Returns:
            True if the command internal was checked, False otherwise
        """
        if not self.create_deploy_command_app(image_name=image_name):
            return False
        ip_address = self.openshift_ops.get_service_ip(service_name=service_name)
        cmd = cmd.replace("<IP>", ip_address)
        for _ in range(timeout):
            output = self.command_app_run(cmd=cmd, return_output=True)
            if expected_output in output:
                return True
            logger.info(
                "Output %s in NOT present in the output of `%s`",
                expected_output,
                cmd,
            )
            time.sleep(3)
        return False

    def check_response_inside_cluster(
        self,
        cmd_to_run: str = None,
        name_in_template: str = "",
        expected_output: str = "",
        port: int = 8080,
        protocol: str = "http",
        response_code: int = 200,
        max_tests: int = 20,
    ) -> bool:
        """
        Check the response inside the cluster.
        Args:
            cmd_to_run: The command to run
            name_in_template: The name of the template
            expected_output: The expected output
            port: The port
            protocol: The protocol
            response_code: The response code
            max_tests: The maximum number of tests
        Returns:
            True if the response inside the cluster was checked, False otherwise
        """
        ip_address = self.openshift_ops.get_service_ip(service_name=name_in_template)
        url = f"{protocol}://{ip_address}:{port}/"
        logger.info("URL address to get internal response is: %s", url)
        if not self.create_deploy_command_app():
            return False
        if cmd_to_run is None:
            cmd_to_run = "curl --connect-timeout 10 -k -s -w '%{http_code}' " + f"{url}"
        # Check if application returns proper HTTP_CODE
        logger.info("Check if HTTP_CODE is valid.")
        for _ in range(max_tests):
            output_code = self.command_app_run(cmd=f"{cmd_to_run}", return_output=True)
            return_code = output_code[-3:]
            try:
                int_ret_code = int(return_code)
                if int_ret_code == response_code:
                    logger.info("HTTP_CODE is VALID %s", int_ret_code)
                    break
            except ValueError:
                logger.info("%s", return_code)
                time.sleep(1)
                continue
            time.sleep(3)
            continue

        cmd_to_run = "curl --connect-timeout 10 -k -s " + f"{url}"
        # Check if application returns proper output
        for count in range(max_tests):
            output_code = self.command_app_run(cmd=f"{cmd_to_run}", return_output=True)
            logger.info(
                "Check if expected output %s is in %s.", expected_output, cmd_to_run
            )
            if expected_output in output_code:
                logger.info("Expected output '%s' is present.", expected_output)
                return True
            logger.info(
                "check_response_inside_cluster: expected_output %s not found in output of %s command. See %s",
                expected_output,
                cmd_to_run,
                output_code,
            )
            time.sleep(5)
        return False

    def check_response_outside_cluster(
        self,
        name_in_template: str = "",
        expected_output: str = "",
        port: int = None,
        protocol: str = "http",
        response_code: int = 200,
    ) -> bool:
        """
        Check the response outside the cluster.
        Args:
            name_in_template: The name of the template
            expected_output: The expected output
            port: The port
            protocol: The protocol
            response_code: The response code
        Returns:
            True if the response outside the cluster was checked, False otherwise
        """
        route_name = self.get_route_url(routes_name=name_in_template)
        logger.info("Route name is %s", route_name)
        url = f"{protocol}://{route_name}"
        for count in range(10):
            logger.info(
                "Let's try to get response from route %s one more time %s", url, count
            )
            response_status = utils.get_response_request(
                url_address=url,
                response_code=response_code,
                expected_str=expected_output,
            )
            if not response_status:
                time.sleep(5)
                continue
            break

        return response_status



#!/usr/bin/env python3

# MIT License
#
# Copyright (c) 2018-2024 Red Hat, Inc.

# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

"""
Tests for DatabaseWrapper class.

This test module verifies the functionality of the DatabaseWrapper class,
which is a Python/PyTest conversion of the bash assert_login_access functions
from mysql-container/test/run and postgresql-container/test/run_test.
"""

import pytest
import subprocess
from unittest.mock import patch

from container_ci_suite.engines.database import DatabaseWrapper


class TestDatabaseWrapperMySQL:
    """Test suite for DatabaseWrapper class with MySQL."""

    def setup_method(self):
        """Set up test fixtures before each test method."""
        self.image_name = "mysql:8.0"
        self.db = DatabaseWrapper(image_name=self.image_name, db_type="mysql")
        self.container_ip = "172.17.0.2"
        self.username = "testuser"
        self.password = "testpass"
        self.database = "testdb"

    def test_init_mysql(self):
        """Test DatabaseWrapper initialization for MySQL."""
        assert self.db.image_name == self.image_name
        assert self.db.db_type == "mysql"

    @patch(
        "container_ci_suite.engines.podman_wrapper.PodmanCLIWrapper.call_podman_command"
    )
    def test_assert_login_access_mysql_success(self, mock_podman):
        """Test assert_login_access for MySQL when login succeeds."""
        mock_podman.return_value = "1\n"

        result = self.db.assert_login_access(
            container_ip=self.container_ip,
            username=self.username,
            password=self.password,
            expected_success=True,
        )

        assert result is True

    @patch(
        "container_ci_suite.engines.podman_wrapper.PodmanCLIWrapper.call_podman_command"
    )
    def test_assert_login_access_mysql_failure_expected(self, mock_podman):
        """Test assert_login_access for MySQL when login fails as expected."""
        mock_podman.side_effect = subprocess.CalledProcessError(1, "cmd")

        result = self.db.assert_login_access(
            container_ip=self.container_ip,
            username=self.username,
            password="wrongpass",
            expected_success=False,
        )

        assert result is True

    @patch(
        "container_ci_suite.engines.podman_wrapper.PodmanCLIWrapper.call_podman_command"
    )
    def test_mysql_cmd(self, mock_podman):
        """Test mysql_cmd method."""
        expected_output = "Query result"
        mock_podman.return_value = expected_output

        result = self.db.mysql_cmd(
            container_ip=self.container_ip,
            username=self.username,
            password=self.password,
        )

        assert result == expected_output
        call_args = mock_podman.call_args[1]["cmd"]
        assert "mysql" in call_args
        assert "--port 3306" in call_args


class TestDatabaseWrapperPostgreSQL:
    """Test suite for DatabaseWrapper class with PostgreSQL."""

    def setup_method(self):
        """Set up test fixtures before each test method."""
        self.image_name = "postgres:13"
        self.db = DatabaseWrapper(image_name=self.image_name, db_type="postgresql")
        self.container_ip = "172.17.0.2"
        self.username = "testuser"
        self.password = "testpass"
        self.database = "testdb"

    def test_init_postgresql(self):
        """Test DatabaseWrapper initialization for PostgreSQL."""
        assert self.db.image_name == self.image_name
        assert self.db.db_type == "postgresql"

    @patch(
        "container_ci_suite.engines.podman_wrapper.PodmanCLIWrapper.call_podman_command"
    )
    def test_assert_login_access_postgresql_success(self, mock_podman):
        """Test assert_login_access for PostgreSQL when login succeeds."""
        mock_podman.return_value = "1"

        result = self.db.assert_login_access(
            container_ip=self.container_ip,
            username=self.username,
            password=self.password,
            expected_success=True,
        )

        assert result is True

    @patch(
        "container_ci_suite.engines.podman_wrapper.PodmanCLIWrapper.call_podman_command"
    )
    def test_assert_login_access_postgresql_failure_expected(self, mock_podman):
        """Test assert_login_access for PostgreSQL when login fails as expected."""
        mock_podman.side_effect = subprocess.CalledProcessError(1, "cmd")

        result = self.db.assert_login_access(
            container_ip=self.container_ip,
            username=self.username,
            password="wrongpass",
            expected_success=False,
        )

        assert result is True

    @patch(
        "container_ci_suite.engines.podman_wrapper.PodmanCLIWrapper.call_podman_command"
    )
    def test_postgresql_cmd(self, mock_podman):
        """Test postgresql_cmd method."""
        expected_output = "Query result"
        mock_podman.return_value = expected_output

        result = self.db.postgresql_cmd(
            container_ip=self.container_ip,
            username=self.username,
            password=self.password,
        )

        assert result == expected_output
        call_args = mock_podman.call_args[1]["cmd"]
        assert "psql" in call_args
        assert "5432" in call_args
        assert "ON_ERROR_STOP=1" in call_args


class TestDatabaseWrapperCommon:
    """Test common functionality across database types."""

    @pytest.mark.parametrize(
        "db_type,image",
        [
            ("mysql", "mysql:8.0"),
            ("mariadb", "mariadb:10.5"),
            ("postgresql", "postgres:13"),
            ("postgres", "postgres:14"),
        ],
    )
    def test_init_different_types(self, db_type, image):
        """Test initialization with different database types."""
        db = DatabaseWrapper(image_name=image, db_type=db_type)
        assert db.image_name == image
        assert db.db_type == db_type.lower()

    @pytest.mark.parametrize(
        "db_type,expected_port",
        [
            ("mysql", 3306),
            ("mariadb", 3306),
            ("postgresql", 5432),
            ("postgres", 5432),
        ],
    )
    @patch(
        "container_ci_suite.engines.podman_wrapper.PodmanCLIWrapper.call_podman_command"
    )
    def test_default_ports(self, mock_podman, db_type, expected_port):
        """Test that default ports are used correctly."""
        mock_podman.return_value = "1"
        db = DatabaseWrapper(image_name="test:latest", db_type=db_type)

        db.test_connection("172.17.0.2", "user", "pass")

        call_args = mock_podman.call_args[1]["cmd"]
        assert str(expected_port) in call_args

    @patch("container_ci_suite.engines.database.time.sleep")
    @patch(
        "container_ci_suite.engines.podman_wrapper.PodmanCLIWrapper.call_podman_command"
    )
    def test_test_connection_with_retries_mysql(self, mock_podman, mock_sleep):
        """Test test_connection with retries for MySQL."""
        # Fail twice, then succeed
        mock_podman.side_effect = [
            subprocess.CalledProcessError(1, "cmd"),
            subprocess.CalledProcessError(1, "cmd"),
            "1\n",
        ]

        db = DatabaseWrapper(image_name="mysql:8.0", db_type="mysql")
        result = db.test_connection(
            container_ip="172.17.0.2",
            username="user",
            password="pass",
            max_attempts=10,
            sleep_time=1,
        )

        assert result is True
        assert mock_podman.call_count == 3
        assert mock_sleep.call_count == 2

    @patch("container_ci_suite.engines.database.time.sleep")
    @patch(
        "container_ci_suite.engines.podman_wrapper.PodmanCLIWrapper.call_podman_command"
    )
    def test_test_connection_with_retries_postgresql(self, mock_podman, mock_sleep):
        """Test test_connection with retries for PostgreSQL."""
        # Fail twice, then succeed
        mock_podman.side_effect = [
            subprocess.CalledProcessError(1, "cmd"),
            subprocess.CalledProcessError(1, "cmd"),
            "1",
        ]

        db = DatabaseWrapper(image_name="postgres:13", db_type="postgresql")
        result = db.test_connection(
            container_ip="172.17.0.2",
            username="user",
            password="pass",
            max_attempts=10,
            sleep_time=1,
        )

        assert result is True
        assert mock_podman.call_count == 3
        assert mock_sleep.call_count == 2

    @patch(
        "container_ci_suite.engines.podman_wrapper.PodmanCLIWrapper.call_podman_command"
    )
    def test_assert_local_access_mysql(self, mock_podman):
        """Test assert_local_access for MySQL."""
        mock_podman.return_value = "1\n"
        db = DatabaseWrapper(image_name="mysql:8.0", db_type="mysql")

        result = db.assert_local_access(container_id="mysql_container")

        assert result is True
        call_args = mock_podman.call_args[1]["cmd"]
        assert "exec mysql_container" in call_args
        assert "mysql -uroot" in call_args

    @patch(
        "container_ci_suite.engines.podman_wrapper.PodmanCLIWrapper.call_podman_command"
    )
    def test_assert_local_access_postgresql(self, mock_podman):
        """Test assert_local_access for PostgreSQL."""
        mock_podman.return_value = "1"
        db = DatabaseWrapper(image_name="postgres:13", db_type="postgresql")

        result = db.assert_local_access(container_id="pg_container")

        assert result is True
        call_args = mock_podman.call_args[1]["cmd"]
        assert "exec -i pg_container" in call_args
        assert "psql" in call_args

    @patch(
        "container_ci_suite.engines.podman_wrapper.PodmanCLIWrapper.call_podman_command"
    )
    def test_assert_local_access_failure(self, mock_podman):
        """Test assert_local_access when access fails."""
        mock_podman.side_effect = subprocess.CalledProcessError(1, "cmd")
        db = DatabaseWrapper(image_name="mysql:8.0", db_type="mysql")

        result = db.assert_local_access(container_id="mysql_container")

        assert result is False


class TestDatabaseWrapperEdgeCases:
    """Test edge cases and error conditions."""

    def test_empty_image_name(self):
        """Test initialization with empty image name."""
        db = DatabaseWrapper(image_name="", db_type="mysql")
        assert db.image_name == ""

    @patch(
        "container_ci_suite.engines.podman_wrapper.PodmanCLIWrapper.call_podman_command"
    )
    def test_special_characters_in_password_mysql(self, mock_podman):
        """Test handling of special characters in password for MySQL."""
        mock_podman.return_value = "1\n"
        db = DatabaseWrapper(image_name="mysql:8.0", db_type="mysql")

        special_password = "p@ss!w0rd#123"
        result = db.assert_login_access(
            container_ip="172.17.0.2", username="user", password=special_password, expected_success=True
        )

        assert result is True

    @patch(
        "container_ci_suite.engines.podman_wrapper.PodmanCLIWrapper.call_podman_command"
    )
    def test_special_characters_in_password_postgresql(self, mock_podman):
        """Test handling of special characters in password for PostgreSQL."""
        mock_podman.return_value = "1"
        db = DatabaseWrapper(image_name="postgres:13", db_type="postgresql")

        special_password = "p@ss!w0rd#123"
        result = db.assert_login_access(
            container_ip="172.17.0.2", username="user", password=special_password, expected_success=True
        )

        assert result is True

    @patch(
        "container_ci_suite.engines.podman_wrapper.PodmanCLIWrapper.call_podman_command"
    )
    def test_custom_port_mysql(self, mock_podman):
        """Test using custom port for MySQL."""
        mock_podman.return_value = "1\n"
        db = DatabaseWrapper(image_name="mysql:8.0", db_type="mysql")

        custom_port = 3307
        result = db.assert_login_access(
            container_ip="172.17.0.2",
            username="user",
            password="pass",
            port=custom_port,
            expected_success=True
        )

        assert result is True
        call_args = mock_podman.call_args[1]["cmd"]
        assert f"--port {custom_port}" in call_args

    @patch(
        "container_ci_suite.engines.podman_wrapper.PodmanCLIWrapper.call_podman_command"
    )
    def test_custom_port_postgresql(self, mock_podman):
        """Test using custom port for PostgreSQL."""
        mock_podman.return_value = "1"
        db = DatabaseWrapper(image_name="postgres:13", db_type="postgresql")

        custom_port = 5433
        result = db.assert_login_access(
            container_ip="172.17.0.2",
            username="user",
            password="pass",
            port=custom_port,
            expected_success=True
        )

        assert result is True
        call_args = mock_podman.call_args[1]["cmd"]
        assert f":{custom_port}/" in call_args


class TestDatabaseWrapperIntegration:
    """Integration tests (skipped by default)."""

    @pytest.mark.integration
    @pytest.mark.skip(reason="Requires actual MySQL container running")
    def test_real_mysql_connection(self):
        """Integration test with real MySQL container."""
        db = DatabaseWrapper(image_name="mysql:8.0", db_type="mysql")

        # These would need to be actual values from a running container
        container_ip = "172.17.0.2"
        username = "root"
        password = "rootpass"

        assert db.test_connection(container_ip, username, password, max_attempts=10)
        assert db.assert_login_access(container_ip, username, password, True)

    @pytest.mark.integration
    @pytest.mark.skip(reason="Requires actual PostgreSQL container running")
    def test_real_postgresql_connection(self):
        """Integration test with real PostgreSQL container."""
        db = DatabaseWrapper(image_name="postgres:13", db_type="postgresql")

        # These would need to be actual values from a running container
        container_ip = "172.17.0.2"
        username = "postgres"
        password = "postgres"

        assert db.test_connection(container_ip, username, password, max_attempts=10)
        assert db.assert_login_access(container_ip, username, password, True)


class TestDatabaseWrapperDocumentation:
    """Test that documentation examples work correctly."""

    @patch(
        "container_ci_suite.engines.podman_wrapper.PodmanCLIWrapper.call_podman_command"
    )
    def test_docstring_example_mysql(self, mock_podman):
        """Test the MySQL example from the docstring."""
        mock_podman.return_value = "1\n"

        db = DatabaseWrapper(image_name="mysql:8.0", db_type="mysql")
        assert db.assert_login_access("172.17.0.2", "user", "pass", True)

    @patch(
        "container_ci_suite.engines.podman_wrapper.PodmanCLIWrapper.call_podman_command"
    )
    def test_docstring_example_postgresql(self, mock_podman):
        """Test the PostgreSQL example from the docstring."""
        mock_podman.return_value = "1"

        db = DatabaseWrapper(image_name="postgres:13", db_type="postgresql")
        assert db.assert_login_access("172.17.0.2", "user", "pass", True)


# Run tests with: pytest tests/test_database_wrapper.py -v
# Run with coverage: pytest tests/test_database_wrapper.py --cov=container_ci_suite.engines.database --cov-report=html
# Run MySQL tests only: pytest tests/test_database_wrapper.py::TestDatabaseWrapperMySQL -v
# Run PostgreSQL tests only: pytest tests/test_database_wrapper.py::TestDatabaseWrapperPostgreSQL -v



#!/bin/env python3

# MIT License
#
# Copyright (c) 2018-2019 Red Hat, Inc.

# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

import subprocess

from flexmock import flexmock
from unittest.mock import patch

from container_ci_suite.openshift import OpenShiftAPI
from container_ci_suite.utils import ContainerTestLibUtils
from container_ci_suite import utils


class TestDeployImageStreamTemplate(object):
    """
    Tests for OpenShiftAPI.deploy_image_stream_template.
    """

    def test_deploy_image_stream_template_returns_false_when_project_not_created(self):
        """
        Short-circuits when project_created is False.
        """
        oc_api = OpenShiftAPI(namespace="ns-test")
        oc_api.project_created = False
        assert (
            oc_api.deploy_image_stream_template("is.json", "tpl.json", "app") is False
        )

    def test_deploy_image_stream_template_returns_false_when_import_is_fails(self):
        """
        Returns False when import_is yields a falsy JSON payload.
        """
        oc_api = OpenShiftAPI(namespace="ns-test")
        oc_api.project_created = True
        oc_api.shared_cluster = False
        flexmock(utils).should_receive("download_template").with_args(
            template_name="is.json"
        ).and_return("/tmp/is.json")
        flexmock(utils).should_receive("download_template").with_args(
            template_name="tpl.json"
        ).and_return("/tmp/tpl.json")
        flexmock(OpenShiftAPI).should_receive("import_is").with_args(
            "/tmp/is.json", name="", skip_check=True
        ).and_return({})

        assert (
            oc_api.deploy_image_stream_template("is.json", "tpl.json", "app") is False
        )

    @patch("container_ci_suite.openshift.time.sleep")
    def test_deploy_image_stream_template_returns_false_when_new_app_raises(
        self, mock_sleep
    ):
        """
        CalledProcessError from oc new-app returns False (after successful import_is).
        """
        oc_api = OpenShiftAPI(namespace="ns-test")
        oc_api.project_created = True
        oc_api.shared_cluster = False
        flexmock(utils).should_receive("download_template").with_args(
            template_name="is.json"
        ).and_return("/tmp/is.json")
        flexmock(utils).should_receive("download_template").with_args(
            template_name="tpl.json"
        ).and_return("/tmp/tpl.json")
        flexmock(OpenShiftAPI).should_receive("import_is").with_args(
            "/tmp/is.json", name="", skip_check=True
        ).and_return({"kind": "List"})
        flexmock(ContainerTestLibUtils).should_receive("run_oc_command").and_raise(
            subprocess.CalledProcessError(1, "oc", output=b"fail")
        )

        assert (
            oc_api.deploy_image_stream_template("is.json", "tpl.json", "myapp")
            is False
        )
        mock_sleep.assert_not_called()

    @patch("container_ci_suite.openshift.time.sleep")
    def test_deploy_image_stream_template_success_without_openshift_args(
        self, mock_sleep
    ):
        """
        Downloads templates, imports IS, runs new-app with NAMESPACE, then sleeps 3s.
        """
        oc_api = OpenShiftAPI(namespace="ns-test")
        oc_api.project_created = True
        oc_api.shared_cluster = False
        flexmock(utils).should_receive("download_template").with_args(
            template_name="imagestreams/httpd.json"
        ).and_return("/tmp/httpd-is.json")
        flexmock(utils).should_receive("download_template").with_args(
            template_name="templates/httpd.json"
        ).and_return("/tmp/httpd-tpl.json")
        flexmock(OpenShiftAPI).should_receive("import_is").with_args(
            "/tmp/httpd-is.json", name="", skip_check=True
        ).and_return({"kind": "List"})
        expected_cmd = (
            "new-app -f /tmp/httpd-tpl.json --name=httpd-app "
            "-p NAMESPACE=ns-test "
        )
        flexmock(ContainerTestLibUtils).should_receive("run_oc_command").with_args(
            expected_cmd,
            json_output=False,
        ).and_return("created").once()

        assert (
            oc_api.deploy_image_stream_template(
                "imagestreams/httpd.json",
                "templates/httpd.json",
                "httpd-app",
            )
            is True
        )
        mock_sleep.assert_called_once_with(3)

    @patch("container_ci_suite.openshift.time.sleep")
    def test_deploy_image_stream_template_success_with_openshift_args(self, mock_sleep):
        """
        Passes openshift_args through get_openshift_args into the new-app command.
        """
        oc_api = OpenShiftAPI(namespace="ns-test")
        oc_api.project_created = True
        oc_api.shared_cluster = False
        flexmock(utils).should_receive("download_template").with_args(
            template_name="is.json"
        ).and_return("/tmp/is.json")
        flexmock(utils).should_receive("download_template").with_args(
            template_name="tpl.json"
        ).and_return("/tmp/tpl.json")
        flexmock(OpenShiftAPI).should_receive("import_is").with_args(
            "/tmp/is.json", name="", skip_check=True
        ).and_return({"kind": "List"})
        # get_openshift_args joins with " -p "; outer f-string adds leading "-p "
        expected_cmd = (
            "new-app -f /tmp/tpl.json --name=app1 -p NAMESPACE=ns-test "
            "-p MEMORY_LIMIT=512Mi -p FOO=bar"
        )
        flexmock(ContainerTestLibUtils).should_receive("run_oc_command").with_args(
            expected_cmd,
            json_output=False,
        ).and_return("created").once()

        assert (
            oc_api.deploy_image_stream_template(
                "is.json",
                "tpl.json",
                "app1",
                openshift_args=["MEMORY_LIMIT=512Mi", "FOO=bar"],
            )
            is True
        )
        mock_sleep.assert_called_once_with(3)

    @patch("container_ci_suite.openshift.time.sleep")
    def test_deploy_image_stream_template_shared_cluster_updates_template(
        self, mock_sleep
    ):
        """
        On shared cluster, update_template_example_file runs on the local template path.
        """
        oc_api = OpenShiftAPI(namespace="ns-test")
        oc_api.project_created = True
        oc_api.shared_cluster = True
        flexmock(utils).should_receive("download_template").with_args(
            template_name="is.json"
        ).and_return("/tmp/is.json")
        flexmock(utils).should_receive("download_template").with_args(
            template_name="tpl.json"
        ).and_return("/tmp/tpl.json")
        flexmock(OpenShiftAPI).should_receive("update_template_example_file").with_args(
            file_name="/tmp/tpl.json"
        ).and_return({}).once()
        flexmock(OpenShiftAPI).should_receive("import_is").with_args(
            "/tmp/is.json", name="", skip_check=True
        ).and_return({"kind": "List"})
        flexmock(ContainerTestLibUtils).should_receive("run_oc_command").with_args(
            "new-app -f /tmp/tpl.json --name=app2 -p NAMESPACE=ns-test ",
            json_output=False,
        ).and_return("ok").once()

        assert (
            oc_api.deploy_image_stream_template("is.json", "tpl.json", "app2") is True
        )
        mock_sleep.assert_called_once_with(3)



# MIT License
#
# Copyright (c) 2018-2019 Red Hat, Inc.
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
import subprocess
import time
import json
import logging

from typing import Any

from container_ci_suite.utils import ContainerTestLibUtils

logger = logging.getLogger(__name__)


class PodmanCLIWrapper(object):
    """
    Podman CLI Wrapper - Class for wrapping podman commands.
    """

    @staticmethod
    def call_podman_command(
        cmd,
        return_output: bool = True,
        ignore_error: bool = False,
        shell: bool = True,
        debug: bool = False,
        stderr=subprocess.STDOUT,
    ):
        """
        Run docker command:

        Args:
            cmd: The command to run
            return_output: Whether to return the output of the command
            ignore_error: Whether to ignore errors
            shell: Whether to run the command in a shell
            debug: Whether to print the command
            stderr: The stderr to use

        Returns:
            The output of the command
        """
        return ContainerTestLibUtils.run_command(
            f"podman {cmd}",
            return_output=return_output,
            ignore_error=ignore_error,
            shell=shell,
            debug=debug,
            stderr=stderr,
        )

    @staticmethod
    def podman_image_exists(image_name: str) -> bool:
        """
        Check if docker image exists or not
        :param image_name: image to check
        :return True: In case if image is present
                False: In case if image is not present
        """
        output = PodmanCLIWrapper.call_podman_command(
            f"images -q {image_name}", ignore_error=True, return_output=True
        )
        return True if output != "" else False

    @staticmethod
    def podman_inspect(field: str, src_image: str) -> str:
        """
        Inspect the container or image.

        Args:
            field: The field to inspect
            src_image: The image to inspect

        Returns:
            The output of the command
        """
        return PodmanCLIWrapper.call_podman_command(f"inspect -f '{field}' {src_image}")

    @staticmethod
    def podman_run_command(cmd, ignore_error: bool = False, return_output: bool = True):
        """
        Run a podman command.

        Args:
            cmd: The command to run
            ignore_error: Whether to ignore errors
            return_output: Whether to return the output of the command
        Returns:
            The output of the command
        """
        return PodmanCLIWrapper.call_podman_command(
            f"run {cmd}", ignore_error=ignore_error, return_output=return_output
        )

    @staticmethod
    def podman_exec_shell_command(
        cid_file_name: str,
        cmd: str,
        used_shell: str = "/bin/bash",
        return_output: bool = True,
        debug: bool = False,
        not_shell: bool = False,
        ignore_error: bool = False,
    ):
        """
        Function executes shell command if image_name is present in system.

        Args:
            cid_file_name: image to check specified by cid_file_name
            cmd: command that will be executed in image
            used_shell: which shell will be used /bin/bash or /bin/sh
            return_output: Whether to return the output of the command
            debug: Whether to print the command
            not_shell: if True, the command will be executed without a shell

        Returns:
            The output of the command
        """
        if not_shell:
            cmd = f"exec {cid_file_name} {cmd}"
        else:
            cmd = f"exec {cid_file_name} {used_shell} -c '{cmd}'"
        logger.info("podman command is: %s", cmd)
        try:
            output = PodmanCLIWrapper.call_podman_command(
                cmd=cmd,
                return_output=return_output,
                debug=debug,
                ignore_error=ignore_error,
            )
        except subprocess.CalledProcessError as cpe:
            logger.error("podman_exec_shell_command: %s failed. See '%s'", cmd, cpe)
            return False
        return output

    @staticmethod
    def podman_run_command_and_remove(
        cid_file_name: str,
        cmd: str,
        return_output: bool = True,
        debug: bool = False,
        ignore_error: bool = False,
    ):
        """
        Run a shell command if the image name is present in the system.

        Args:
            cid_file_name: image to check specified by cid_file_name
            cmd: command that will be executed in image
            return_output: Whether to return the output of the command
            debug: Whether to print the command
            ignore_error: Whether to ignore errors

        Returns:
            True if the command was executed successfully, False otherwise
            The output of the command
        """
        cmd = f"run --rm {cid_file_name} /bin/bash -c '{cmd}'"
        logger.info("podman command is: '%s'", cmd)
        try:
            output = PodmanCLIWrapper.call_podman_command(
                cmd=cmd,
                return_output=return_output,
                ignore_error=ignore_error,
                debug=debug,
            )
        except subprocess.CalledProcessError as cpe:
            logger.error("podman_run_command_and_remove: %s failed. See '%s'", cmd, cpe)
            return False
        return output

    @staticmethod
    def podman_get_user_id(src_image, user):
        """
        Get the user ID of a user in a container.

        Args:
            src_image: The image to get the user ID from
            user: The user to get the user ID of

        Returns:
            The user ID of the user
            The output of the command
        """
        return PodmanCLIWrapper.call_podman_command(
            f"--rm {src_image} /bin/bash -c 'id -u {user}' 2>/dev/null"
        ).strip()

    @staticmethod
    def podman_pull_image(image_name: str, loops: int = 10) -> bool:
        """
        Function checks if image_name is present in system.
        In case it isn't, try to pull it for specific count of loops
        Default is 10.

        Args:
            image_name: The name of the image to pull
            loops: The number of loops to try to pull the image

        Returns:
            True if the image was pulled successfully, False otherwise
        """
        if PodmanCLIWrapper.podman_image_exists(image_name=image_name):
            logger.info("Pulled image already exists.")
            return True
        for loop in range(loops):
            try:
                ret_val = PodmanCLIWrapper.call_podman_command(
                    cmd=f"pull {image_name}", return_output=False
                )
            except subprocess.CalledProcessError:
                ret_val = 1
            if ret_val == 0 and PodmanCLIWrapper.podman_image_exists(
                image_name=image_name
            ):
                return True
            PodmanCLIWrapper.call_podman_command("images", return_output=True)
            logger.error(
                "Pulling of image %s failed. Let's wait %s seconds and try again.",
                image_name,
                loop * 5,
            )
            time.sleep(loop * 5)
        return False

    @staticmethod
    def podman_inspect_ip_address(container_id: str) -> Any:
        """
        Get the IP address of a container.

        Args:
            container_id: The ID of the container to get the IP address of

        Returns:
            The IP address of the container
        """
        output = PodmanCLIWrapper.call_podman_command(f"inspect {container_id}")

        json_output = json.loads(output)
        if len(json_output) == 0:
            return None
        if "NetworkSettings" not in json_output[0]:
            return None
        return json_output[0]["NetworkSettings"]["IPAddress"]

    @staticmethod
    def podman_exit_status(image_name: str) -> str:
        """
        Get the exit status of a container.

        Args:
            image_name: The name of the image to get the exit status of

        Returns:
            The exit status of the container
        """
        return PodmanCLIWrapper.call_podman_command(
            cmd=f"inspect --format='{{{{.State.ExitCode}}}}' {image_name}",
            return_output=True,
        ).strip()

    @staticmethod
    def podman_get_user(image_name: str) -> Any:
        """
        Get the user of a container.

        Args:
            image_name: The name of the image to get the user of

        Returns:
            The user of the container
        """
        output = PodmanCLIWrapper.call_podman_command(f"inspect {image_name}")

        json_output = json.loads(output)
        if len(json_output) == 0:
            return None
        if "Config" not in json_output[0]:
            return None
        return json_output[0]["Config"]["User"]

    @staticmethod
    def podman_get_file_content(cid_file_name: str, filename: str) -> str:
        """
        Get the content of a file in a container.

        Args:
            cid_file_name: The name of the container to get the file content of
            filename: The name of the file to get the content of

        Returns:
            The content of the file
        """
        return PodmanCLIWrapper.call_podman_command(
            cmd=f"exec {cid_file_name} cat {filename}"
        )

    @staticmethod
    def podman_logs(
        container_id: str,
    ):
        """
        Get the logs of a container.

        Args:
            container_id: The ID of the container to get the logs of

        Returns:
            The logs of the container
        """
        return PodmanCLIWrapper.call_podman_command(f"logs {container_id}")



#!/usr/bin/env python3

# MIT License
#
# Copyright (c) 2018-2024 Red Hat, Inc.

# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

"""
Database Testing Wrapper Module

This module provides database-specific testing functionality for container tests.
It includes utilities for testing database connections, login access, and other
database-related operations for MySQL/MariaDB and PostgreSQL containers.

Converted from bash functions in:
- mysql-container/test/run (lines 219-239)
- postgresql-container/test/run_test (lines 198-218)
"""

import logging
import re
import subprocess
import time
from typing import Optional, Literal, Union, Dict
from enum import Enum
from urllib.parse import quote, urlencode

from container_ci_suite.engines.podman_wrapper import PodmanCLIWrapper

logger = logging.getLogger(__name__)


class DatabaseType(Enum):
    """Supported database types."""

    MYSQL = "mysql"
    MARIADB = "mariadb"
    POSTGRESQL = "postgresql"
    POSTGRES = "postgres"


class DatabaseWrapper:
    """
    Database testing wrapper class.

    This class provides methods for testing MySQL/MariaDB and PostgreSQL database
    containers, including connection testing, login access verification, and other
    database-specific operations.

    This is a Python/PyTest conversion of the bash assert_login_access functions from:
    - mysql-container/test/run (lines 219-239)
    - postgresql-container/test/run_test (lines 198-218)

    Attributes:
        image_name: Name of the database container image to test
        db_type: Type of database (mysql, mariadb, postgresql)

    Example:
        >>> # MySQL
        >>> db = DatabaseWrapper(image_name="mysql:8.0", db_type="mysql")
        >>> assert db.assert_login_access("172.17.0.2", "user", "pass", expected_success=True)

        >>> # PostgreSQL
        >>> db = DatabaseWrapper(image_name="postgres:13", db_type="postgresql")
        >>> assert db.assert_login_access("172.17.0.2", "user", "pass", expected_success=True)
    """

    def __init__(
        self,
        image_name: str,
        db_type: Literal["mysql", "mariadb", "postgresql", "postgres"] = "mysql",
    ):
        """
        Initialize the DatabaseWrapper.

        Args:
            image_name: Name of the database container image
            db_type: Type of database (mysql, mariadb, postgresql, postgres)
        """
        self.image_name = image_name
        self.db_type = db_type.lower()
        logger.debug(
            "DatabaseWrapper initialized with image: %s, type: %s",
            image_name,
            self.db_type,
        )

    def wait_for_database(
        self,
        container_id: str,
        command: str,
        max_attempts: int = 10,
        sleep_time: int = 3,
        not_shell: bool = True,
    ) -> bool:
        """
        Wait for the database to be ready.
        Args:
            container_id: Container ID or name
            command: Command to execute to test if the database is ready
            max_attempts: Maximum number of attempts to wait for the database to be ready
            sleep_time: Time to sleep between attempts
            not_shell: Whether to execute the command without a shell (default: True)
        Returns:
            True if database is ready, False otherwise
        """
        logger.debug("Waiting for database to be ready...")
        logger.debug("Container ID: %s", container_id)
        logger.debug("Command: %s", command)
        logger.debug("Max attempts: %s", max_attempts)
        logger.debug("Sleep time: %s", sleep_time)
        for attempt in range(1, max_attempts + 1):
            try:
                output = PodmanCLIWrapper.podman_exec_shell_command(
                    cid_file_name=container_id, cmd=command, not_shell=not_shell
                )
                if isinstance(output, bool) and not output:
                    logger.debug(
                        "Database not ready, attempt %s, retrying... (output: '%s')",
                        attempt,
                        output,
                    )
                    time.sleep(sleep_time)
                    continue
                if isinstance(output, str) and output.strip() == "":
                    logger.info("Database is ready (output: %s)", output)
                    return True
            except subprocess.CalledProcessError as cpe:
                logger.error("Error waiting for database: %s", cpe)
            time.sleep(sleep_time)
        logger.error("Database not ready after %s attempts", max_attempts)
        return False

    def assert_login_access(
        self,
        container_ip: str,
        username: str,
        password: str,
        expected_success: bool,
        database: str = "db",
        port: int = None,
    ) -> bool:
        """
        Assert that login access works as expected for MySQL/MariaDB or PostgreSQL.

        This function tests whether a user can successfully log in to a database
        container with the given credentials. It verifies that the login either
        succeeds or fails as expected.

        Automatically detects database type and uses appropriate command.

        Args:
            container_ip: IP address of the container
            username: Username to test login with
            password: Password to test login with
            expected_success: Whether login should succeed (True) or fail (False)
            database: Database name to connect to (default: "db")
            port: Port number (default: 3306 for MySQL, 5432 for PostgreSQL)

        Returns:
            True if login behavior matches expectations, False otherwise

        Example:
            >>> # MySQL - test valid login
            >>> db = DatabaseWrapper(image_name="mysql:8.0", db_type="mysql")
            >>> assert db.assert_login_access("172.17.0.2", "user", "pass", True)

            >>> # PostgreSQL - test invalid login
            >>> db = DatabaseWrapper(image_name="postgres:13", db_type="postgresql")
            >>> assert db.assert_login_access("172.17.0.2", "user", "wrong", False)
        """
        logger.info(
            "Testing %s login as %s:%s; expected_success=%s",
            self.db_type,
            username,
            password,
            expected_success,
        )

        try:
            if self.db_type in ["postgresql", "postgres"]:
                success = self._test_postgresql_login(
                    container_ip, username, password, database, port
                )
            else:  # mysql or mariadb
                success = self._test_mysql_login(
                    container_ip, username, password, database, port
                )

            if success and expected_success:
                logger.info("    %s(%s) access granted as expected", username, password)
                return True
            elif not success and not expected_success:
                logger.info("    %s(%s) access denied as expected", username, password)
                return True
            else:
                logger.error("    %s(%s) login assertion failed", username, password)
                return False

        except Exception as e:
            logger.error("Error during login test: %s", e)
            if not expected_success:
                logger.info("    %s(%s) access denied as expected", username, password)
                return True
            else:
                logger.error("    %s(%s) login assertion failed", username, password)
                return False

    def _test_mysql_login(
        self,
        container_ip: str,
        username: str,
        password: str,
        database: str = "db",
        port: int = None,
    ) -> bool:
        """
        Test MySQL/MariaDB login.

        Bash equivalent:
        mysql_cmd "$container_ip" "$USER" "$PASS" -e 'SELECT 1;' | grep -q -e 1
        """
        try:
            str_port = "--port " + str(port) if port else ""
            cmd = (
                f"run --rm {self.image_name} mysql "
                f"--host {container_ip} {str_port} "
                f"-u{username} -p{password} "
                f"-e 'SELECT 1;' {database}"
            )

            output = PodmanCLIWrapper.call_podman_command(cmd=cmd, return_output=True)

            # Check if the output contains "1" which indicates successful query
            return "1" in output

        except subprocess.CalledProcessError:
            return False

    def _test_postgresql_login(
        self,
        container_ip: str,
        username: str,
        password: str,
        database: str = "db",
        port: int = None,
    ) -> bool:
        """
        Test PostgreSQL login.

        Bash equivalent:
        docker run --rm -e PGPASSWORD="$PASS" "$IMAGE_NAME" psql -v ON_ERROR_STOP=1 \
          "postgresql://$PGUSER@$CONTAINER_IP:5432/${DB-db}" -At -c 'SELECT 1;'
        """
        try:
            output = self.postgresql_cmd(
                container_ip,
                username,
                password,
                container_id=self.image_name,
                database=database,
                port=port if port else 5432,
                extra_args="-At",
                sql_command="-c 'SELECT 1;'",
            )
            # PostgreSQL returns "1" on successful query
            return "1" in output.strip()

        except subprocess.CalledProcessError:
            return False

    def mysql_cmd(
        self,
        container_ip: str,
        username: str,
        password: str,
        database: str = "db",
        port: int = 3306,
        extra_args: str = "",
        sql_command: Optional[str] = None,
        container_id: Optional[str] = None,
        podman_run_command: Optional[str] = "run --rm",
    ) -> str:
        """
        Execute a MySQL command against a container.

        This is a Python equivalent of the bash mysql_cmd function:
        ```bash
        function mysql_cmd() {
          local container_ip="$1"; shift
          local login="$1"; shift
          local password="$1"; shift
          docker run --rm ${CONTAINER_EXTRA_ARGS:-} "$IMAGE_NAME" mysql \\
            --host "$container_ip" -u"$login" -p"$password" "$@" db
        }
        ```

        Args:
            container_ip: IP address of the MySQL container
            username: MySQL username
            password: MySQL password
            database: Database name (default: "db")
            port: Port number (default: 3306)
            extra_args: Additional arguments to pass to mysql command
            sql_command: SQL command to execute (e.g., "-e 'SELECT 1;'")
            container_id: The container ID (usually the image name)
            podman_run_command: Podman run command to use (default: "run --rm")
        Returns:
            Command output as string

        Raises:
            subprocess.CalledProcessError: If the command fails

        Example:
            >>> db = DatabaseWrapper(image_name="mysql:8.0", db_type="mysql")
            >>> output = db.mysql_cmd("172.17.0.2", "user", "pass",
            ...                       sql_command="-e 'SELECT 1;'")
        """
        if not container_id:
            container_id = self.image_name
        if not sql_command:
            sql_command = "-e 'SELECT 1;'"
        cmd_parts = [
            podman_run_command,
            container_id,
            "mysql",
            f"--host {container_ip}",
            f"--port {port}",
            f"-u{username}",
            f"-p{password}",
        ]

        if extra_args:
            cmd_parts.append(extra_args)

        if sql_command:
            cmd_parts.append(sql_command)

        cmd_parts.append(database)

        cmd = " ".join(cmd_parts)
        logging.debug("Executing command: %s", cmd)

        return PodmanCLIWrapper.call_podman_command(
            cmd=cmd, return_output=True, ignore_error=False
        )

    def postgresql_cmd(
        self,
        container_ip: str,
        username: str,
        password: str,
        container_id: Optional[str] = None,
        database: str = "db",
        uri_params: Dict[str, str] = None,
        port: int = 5432,
        extra_args: str = "",
        sql_command: Optional[str] = "",
        podman_run_command: Optional[str] = "run --rm",
        docker_args: str = "",
    ) -> str:
        """
        Execute a PostgreSQL command against a container.

        This is a Python equivalent of the bash postgresql_cmd function:
        ```bash
        function postgresql_cmd() {
          docker run --rm -e PGPASSWORD="$PASS" "$IMAGE_NAME" psql -v ON_ERROR_STOP=1 \\
            "postgresql://$PGUSER@$CONTAINER_IP:5432/${DB-db}" "$@"
        }
        ```

        Args:
            container_ip: IP address of the PostgreSQL container
            username: PostgreSQL username
            password: PostgreSQL password
            container_id: Container ID
            database: Database name (default: "db")
            uri_params: Extra parameters in the postgreSQL connection URI
            port: Port number (default: 5432)
            extra_args: Additional arguments to pass to psql command
            sql_command: SQL command to execute (e.g., "-c 'SELECT 1;'")
            podman_run_command: Podman run command to use (default: "run --rm")
            docker_args: Docker arguments to pass to podman run command
        Returns:
            Command output as string

        Raises:
            subprocess.CalledProcessError: If the command fails

        Example:
            >>> db = DatabaseWrapper(image_name="postgres:13", db_type="postgresql")
            >>> output = db.postgresql_cmd("172.17.0.2", "user", "pass",
            ...                            sql_command="-c 'SELECT 1;'")
        """
        encoded_username = quote(username)
        encoded_database = quote(database)
        encoded_params = f"?{urlencode(uri_params)}" if uri_params else ""
        connection_string = f"postgresql://{encoded_username}@{container_ip}:{port}/{encoded_database}{encoded_params}"
        if not container_id:
            container_id = self.image_name
        cmd_parts = [
            podman_run_command,
            docker_args,
            f'-e PGPASSWORD="{password}"',
            container_id,
            "psql",
            "-v ON_ERROR_STOP=1",
            connection_string,
            extra_args,
            sql_command,
        ]

        cmd = " ".join(cmd_parts)

        return PodmanCLIWrapper.call_podman_command(cmd=cmd, return_output=True)

    def test_connection(
        self,
        container_ip: str,
        username: str,
        password: str,
        database: str = "db",
        uri_params: Dict[str, str] = None,
        max_attempts: int = 60,
        sleep_time: int = 3,
        sql_cmd: Optional[str] = None,
    ) -> bool:
        """
        Test database connection with retries.

        This function attempts to connect to a database container multiple times,
        waiting between attempts. Useful for testing container startup.
        Works for both MySQL and PostgreSQL.

        Args:
            container_ip: IP address of the container
            username: Database username
            password: Database password
            database: Database name (default: "db")
            max_attempts: Maximum number of connection attempts (default: 60)
            sleep_time: Seconds to wait between attempts (default: 3)
            sql_cmd: SQL command to execute (e.g., "SELECT 1;")
        Returns:
            True if connection successful, False otherwise

        Example:
            >>> db = DatabaseWrapper(image_name="mysql:8.0", db_type="mysql")
            >>> if db.test_connection("172.17.0.2", "user", "pass"):
            ...     print("Database is ready!")
        """
        logger.info("Testing %s connection to %s...", self.db_type, container_ip)
        logger.info("Trying to connect...")
        for attempt in range(1, max_attempts + 1):
            try:
                if self.db_type in ["postgresql", "postgres"]:
                    sql_cmd = sql_cmd or "-At -c 'SELECT 1;'"
                    return_output = self.postgresql_cmd(
                        container_ip=container_ip,
                        username=username,
                        password=password,
                        database=database,
                        uri_params=uri_params,
                        sql_command=sql_cmd,
                    )
                else:  # mysql or mariadb
                    sql_cmd = sql_cmd or "-e 'SELECT 1;'"
                    return_output = self.mysql_cmd(
                        container_ip=container_ip,
                        username=username,
                        password=password,
                        database=database,
                        sql_command=sql_cmd,
                    )
                logging.debug("Output: %s", return_output)
                logger.info("Connection successful on attempt %s", attempt)
                return True

            except subprocess.CalledProcessError:
                if attempt < max_attempts:
                    logger.debug("Attempt %s failed, retrying...", attempt)
                    time.sleep(sleep_time)
                else:
                    logger.error("Failed to connect after %s attempts", max_attempts)
                    return False

        return False

    def assert_local_access(self, container_id: str, username: str = None) -> bool:
        """
        Assert that local access to database works from inside the container.

        This tests if the database command works when executed inside the container.
        Works for both MySQL and PostgreSQL.

        Bash equivalent (MySQL):
        ```bash
        docker exec $(ct_get_cid "$id") bash -c 'mysql -uroot <<< "SELECT 1;"'
        ```

        Bash equivalent (PostgreSQL):
        ```bash
        docker exec -i $(get_cid "$id") bash -c psql <<< "SELECT 1;"
        ```

        Args:
            container_id: Container ID or name
            username: Username to test with (default: "root" for MySQL, None for PostgreSQL)

        Returns:
            True if local access works, False otherwise

        Example:
            >>> db = DatabaseWrapper(image_name="mysql:8.0", db_type="mysql")
            >>> assert db.assert_local_access("mysql_container")
        """
        try:
            if self.db_type in ["postgresql", "postgres"]:
                # PostgreSQL doesn't need username for local access
                cmd = f"exec -i {container_id} bash -c 'psql <<< \"SELECT 1;\"'"
            else:  # mysql or mariadb
                user = username or "root"
                cmd = f"exec {container_id} bash -c 'mysql -u{user} <<< \"SELECT 1;\"'"

            output = PodmanCLIWrapper.call_podman_command(cmd=cmd, return_output=True)

            if "1" in output:
                logger.info("    Local access granted as expected")
                return True
            else:
                logger.error("    Local access assertion failed")
                return False

        except subprocess.CalledProcessError:
            logger.error("    Local access assertion failed")
            return False

    def run_sql_command(
        self,
        username: Optional[str] = None,
        password: Optional[str] = None,
        container_ip: str = None,
        port: int = 3306,
        sql_cmd: Optional[Union[list[str], str]] = None,
        database: str = "db",
        max_attempts: int = 10,
        sleep_time: int = 3,
        container_id: Optional[str] = None,
        docker_args: Optional[Union[list[str], str]] = "",
        podman_run_command: Optional[str] = "run --rm",
        ignore_error: bool = False,
        expected_output: Optional[str] = None,
        use_bash: bool = False,
    ) -> str | bool:
        """
        Run a database command inside the container.

        Bash equivalent:
        ```bash
        docker exec -i $(get_cid "$id") bash -c psql <<< "SELECT 1;"
        ```

        Args:
            username: Username to test with (default: "root" for MySQL, None for PostgreSQL)
            password: Password to test with
            container_ip: IP address of the container
            sql_cmd: SQL command to execute (e.g., "SELECT 1;")
            database: Database name (default: "db")
            port: Port number (default: 3306 for MySQL, 5432 for PostgreSQL)
            max_attempts: Maximum number of attempts (default: 60)
            sleep_time: Time to sleep between attempts (default: 3)
            container_id: Container ID or name
            docker_args: Additional arguments for the `podman run` command
            podman_run_command: Podman run command to use (default: "run --rm")
            ignore_error: Ignore error and return output (default: False)
            expected_output: Expected output of the command (default: None)
            use_bash: Set to true to execute each command in `sql_cmd` in bash
        Returns:
            Command output as string or False if command failed
        """
        if not container_id:
            container_id = self.image_name
        if not sql_cmd:
            sql_cmd = "SELECT 1;"
        if isinstance(sql_cmd, str):
            sql_cmd = [sql_cmd]
        if isinstance(docker_args, list):
            docker_args = " ".join(docker_args)
        logger.debug(
            "Podman run command: %s with image: %s", podman_run_command, container_id
        )
        logger.debug("Database type: %s", self.db_type)
        logger.debug("Docker arguments: %s", docker_args)
        logger.debug("SQL command: %s", sql_cmd)
        logger.debug("Database: %s", database)
        logger.debug("Username: %s", username)
        logger.debug("Password: %s", password)
        logger.debug("Container IP: %s", container_ip)
        logger.debug("Port: %s", port)
        logger.debug("Max attempts: %s", max_attempts)
        logger.debug("Sleep time: %s", sleep_time)
        return_output = None
        for cmd in sql_cmd:
            if use_bash:
                cmd = f'bash -c "{cmd}"'
            for attempt in range(1, max_attempts + 1):
                if self.db_type in ["postgresql", "postgres"]:
                    try:
                        return_output = self.postgresql_cmd(
                            container_ip=container_ip,
                            username=username,
                            password=password,
                            database=database,
                            sql_command=cmd,
                            container_id=container_id,
                            podman_run_command=podman_run_command,
                            docker_args=docker_args,
                        )
                        logger.info("PostgreSQL return output: %s", return_output)
                    except subprocess.CalledProcessError as cpe:
                        # In case of ignore_error, we return the output
                        # This is useful for commands that are expected to fail, like wrong login
                        if ignore_error:
                            return_output = cpe.output
                        else:
                            logger.error(
                                "Failed to execute command, output: %s, error: %s",
                                cpe.output,
                                cpe.stderr,
                            )
                            return False
                else:
                    try:
                        return_output = self.mysql_cmd(
                            container_ip=container_ip,
                            username=username,
                            password=password,
                            database=database,
                            sql_command=f"-e '{cmd}'",
                            container_id=container_id,
                            podman_run_command=podman_run_command,
                        )
                        logger.info("MySQL return output: '%s'", return_output)
                    except subprocess.CalledProcessError as cpe:
                        # In case of ignore_error, we return the output
                        # This is useful for commands that are expected to fail, like wrong login
                        if ignore_error:
                            return_output = cpe.output
                        else:
                            logger.error(
                                "Failed to execute command, output: %s, error: %s",
                                cpe.output,
                                cpe.stderr,
                            )
                            return False
                if expected_output is None:
                    logger.info(
                        "Command executed successfully without checking for expected output on attempt %s"
                        % attempt
                    )
                    break
                if re.search(expected_output, return_output):
                    logger.info("Command executed successfully on attempt %s", attempt)
                    break
                else:
                    logger.debug("Expected output not found in return output")
                if attempt < max_attempts:
                    time.sleep(sleep_time)
                else:
                    return False
        if return_output:
            logger.info("All commands executed successfully")
            logger.debug("Output:\n'%s'", return_output)
            return return_output
        return False



"""
Integration tests for ContainerTestLib class.
These tests require Docker to be available and may be slower.
"""

import pytest

from requests import Response
from flexmock import flexmock
from unittest.mock import patch
from subprocess import CalledProcessError

from container_ci_suite.container_lib import ContainerTestLib
from container_ci_suite.engines.container import ContainerImage


@pytest.mark.integration
class TestContainerIntegration:
    """Integration tests for container operations."""

    def setup_method(self):
        self.lib = ContainerTestLib()

    def test_create_container_success(self, mock_docker_commands, mock_file_operations):
        """Test successful container creation."""
        self.lib.image_name = "test:latest"

        # Mock the CID file creation
        cid_file_path = str(self.lib.cid_file_dir / "test_container")
        mock_file_operations[cid_file_path] = "container123"

        with patch.object(ContainerImage, "wait_for_cid", return_value=True):
            result = self.lib.create_container("test_container", "sleep 60")
            assert result is True

    def test_ct_create_container_failure(self):
        """Test failed container creation."""
        self.lib.image_name = "test:latest"

        with patch(
            "container_ci_suite.utils.ContainerTestLibUtils.run_command"
        ) as mock_cmd:
            mock_cmd.side_effect = CalledProcessError(1, "docker run")
            result = self.lib.create_container("test_container", "sleep 60")
            assert result is False

    # def test_ct_assert_container_creation_fails_success(
    #     self, mock_docker_commands, mock_file_operations
    # ):
    #     """Test assertion that container creation should fail."""

    #     # Mock container that exits with non-zero status
    #     with (
    #         patch.object(self.lib, "create_container", return_value=True),
    #         patch.object(self.lib, "get_cid", return_value="container123"),
    #         patch.object(ContainerImage, "is_container_running", return_value=False),
    #         patch(
    #             "container_ci_suite.utils.ContainerTestLibUtils.run_command"
    #         ) as mock_cmd,
    #     ):
    #         # Mock exit status check
    #         mock_cmd.return_value = "1"  # Non-zero exit code

    #         result = self.lib.assert_container_creation_fails(
    #             cid_file_name="test_container",
    #             container_args="--invalid-arg",
    #             command="sleep 1",
    #         )
    #         assert result is True


@pytest.mark.integration
class TestImageIntegration:
    """Integration tests for image operations."""

    def setup_method(self):
        self.lib = ContainerTestLib()

    def test_image_build_and_cleanup(self, temp_dir, sample_dockerfile_content):
        """Test building image and cleanup."""

        # Create a test Dockerfile
        dockerfile = temp_dir / "Dockerfile"
        dockerfile.write_text(sample_dockerfile_content)

        with patch(
            "container_ci_suite.utils.ContainerTestLibUtils.run_command"
        ) as mock_cmd:
            mock_cmd.return_value = "Successfully built abc123\nabc123"

            result = self.lib.build_image_and_parse_id(str(dockerfile), str(temp_dir))
            assert result is True
            assert self.lib.app_image_id == "abc123"

    def test_binary_found_from_dockerfile(self):
        """Test binary availability check from Dockerfile."""
        self.lib.image_name = "test:latest"

        with patch(
            "container_ci_suite.utils.ContainerTestLibUtils.run_command"
        ) as mock_cmd:
            mock_cmd.return_value = "Successfully built def456\ndef456"

            result = self.lib.binary_found_from_df("node", "/usr/bin")
            assert result is True


@pytest.mark.integration
class TestS2IIntegration:
    """Integration tests for S2I operations."""

    def test_s2i_usage(self, mock_docker_commands):
        """Test S2I usage command."""
        lib = ContainerTestLib(image_name="test:latest")
        result = lib.s2i_usage()
        assert result == ""  # Mocked to return empty string


@pytest.mark.integration
class TestEnvironmentIntegration:
    """Integration tests for environment checking."""

    def setup_method(self):
        self.lib = ContainerTestLib()

    def test_check_exec_env_vars_success(self, sample_environment_vars):
        """Test environment variable checking between run and exec."""
        self.lib.image_name = "test:latest"

        env_output = "\n".join([f"{k}={v}" for k, v in sample_environment_vars.items()])

        with (
            patch(
                "container_ci_suite.utils.ContainerTestLibUtils.run_command"
            ) as mock_cmd,
            patch.object(self.lib, "create_container", return_value=True),
            patch.object(self.lib, "get_cid", return_value="container123"),
        ):
            mock_cmd.return_value = env_output

            result = self.lib.check_exec_env_vars()
            assert result is True

    def test_ct_check_scl_enable_vars_success(self, sample_environment_vars):
        """Test SCL environment variable checking."""
        self.lib.image_name = "test:latest"

        with patch(
            "container_ci_suite.utils.ContainerTestLibUtils.run_command"
        ) as mock_cmd:
            # Mock X_SCLS output
            mock_cmd.side_effect = [
                "nodejs16",  # X_SCLS value
                "\n".join(
                    [f"{k}={v}" for k, v in sample_environment_vars.items()]
                ),  # env output
                "\n".join(
                    [f"{k}={v}{v}" for k, v in sample_environment_vars.items()]
                ),  # scl enable env
            ]

            result = self.lib.check_scl_enable_vars()
            assert result is True


@pytest.mark.integration
@pytest.mark.network
class TestNetworkIntegration:
    """Integration tests requiring network access."""

    def setup_method(self):
        """
        Setup the test environment.
        """
        self.lib = ContainerTestLib()

    def test_response_success(self):
        """Test HTTP response testing."""
        response = Response()
        response.status_code = 200
        response.reason = "Welcome to the app200"
        response.headers = {"Content-Type": "text/html"}
        flexmock(self.lib).should_receive("_get_response").and_return(response)
        result = self.lib.test_response(
            url="http://localhost",
            port=8080,
            expected_code=200,
            expected_output="Welcome",
            max_attempts=1,
            debug=True,
        )
        assert result is True

    def test_response_failure(self):
        """Test HTTP response testing with failure."""
        response = Response()
        response.status_code = 404
        response.reason = "Error404"
        response.headers = {"Content-Type": "text/html"}
        flexmock(self.lib).should_receive("_get_response").and_return(response)
        result = self.lib.test_response(
            url="http://localhost",
            port=8080,
            expected_code=404,
            expected_output="Welcome",
            max_attempts=1,
            debug=True,
        )
        assert result is False


@pytest.mark.integration
class TestNpmIntegration:
    """Integration tests for NPM functionality."""

    def setup_method(self):
        """
        Setup the test environment.
        """
        self.lib = ContainerTestLib()

    def test_npm_works_success(self, temp_dir):
        """Test NPM functionality check."""
        self.lib.image_name = "test:latest"

        with (
            patch(
                "container_ci_suite.utils.ContainerTestLibUtils.run_command"
            ) as mock_cmd,
            patch.object(ContainerImage, "wait_for_cid", return_value=True),
            patch(
                "container_ci_suite.utils.get_file_content", return_value="container123"
            ),
        ):
            mock_cmd.side_effect = [
                "8.19.2",  # npm --version
                "",  # docker run for test container
                "npm install success",  # npm install jquery
                "",  # docker stop
            ]

            result = self.lib.npm_works()
            assert result is True

    def test_ct_npm_works_version_failure(self):
        """Test NPM functionality check with version failure."""
        self.lib.image_name = "test:latest"

        with patch(
            "container_ci_suite.utils.ContainerTestLibUtils.run_command"
        ) as mock_cmd:
            mock_cmd.side_effect = CalledProcessError(1, "npm --version")

            result = self.lib.npm_works()
            assert result is False


@pytest.mark.integration
class TestDocumentationIntegration:
    """Integration tests for documentation checking."""

    def setup_method(self):
        self.lib = ContainerTestLib()

    def test_doc_content_old_success(self):
        """Test documentation content checking."""
        doc_content = """.TH NODEJS 1 "nodejs container"
.PP
This is a nodejs container
.SH DESCRIPTION
Node.js runtime for applications
"""

        with patch(
            "container_ci_suite.utils.ContainerTestLibUtils.run_command"
        ) as mock_cmd:
            mock_cmd.return_value = doc_content

            result = self.lib.doc_content_old(["nodejs", "container"])
            assert result is True

    def test_doc_content_old_missing_string(self):
        """Test documentation content checking with missing string."""
        doc_content = """.TH NODEJS 1 "nodejs container"
.PP
This is a nodejs container
"""
        with patch(
            "container_ci_suite.utils.ContainerTestLibUtils.run_command"
        ) as mock_cmd:
            mock_cmd.return_value = doc_content

            result = self.lib.doc_content_old(["missing_string"])
            assert result is False

    def test_doc_content_old_invalid_format(self):
        """Test documentation content checking with invalid format."""
        doc_content = "This is not a proper troff format"
        with patch(
            "container_ci_suite.utils.ContainerTestLibUtils.run_command"
        ) as mock_cmd:
            mock_cmd.return_value = doc_content

            result = self.lib.doc_content_old(["nodejs"])
            assert result is False


@pytest.mark.integration
class TestCertificateIntegration:
    """Integration tests for certificate operations."""

    def setup_method(self):
        self.lib = ContainerTestLib()

    def test_gen_self_signed_cert_pem(self, temp_dir):
        """Test self-signed certificate generation."""
        with patch(
            "container_ci_suite.utils.ContainerTestLibUtils.run_command"
        ) as mock_cmd:
            mock_cmd.return_value = ""  # Successful command execution

            result = self.lib.gen_self_signed_cert_pem(str(temp_dir), "test")
            assert result is True

            # Check that openssl commands were called
            assert mock_cmd.call_count >= 2

    def test_gen_self_signed_cert_pem_failure(self, temp_dir):
        """Test self-signed certificate generation failure."""
        with patch(
            "container_ci_suite.utils.ContainerTestLibUtils.run_command"
        ) as mock_cmd:
            mock_cmd.side_effect = CalledProcessError(1, "openssl")

            result = self.lib.gen_self_signed_cert_pem(str(temp_dir), "test")
            assert result is False



"""
Unit tests for build_as_df_build_args and related functions.
"""

from pathlib import Path
from unittest.mock import patch
from subprocess import CalledProcessError

from container_ci_suite.container_lib import ContainerTestLib


class TestBuildAsDfBuildArgs:
    """Test build_as_df_build_args function."""

    def setup_method(self):
        self.lib = ContainerTestLib()

    @patch("container_ci_suite.container_lib.ContainerTestLib.build_image_and_parse_id")
    @patch("container_ci_suite.container_lib.ContainerTestLib.get_uid_from_image")
    @patch("container_ci_suite.container_lib.PodmanCLIWrapper.call_podman_command")
    @patch("container_ci_suite.container_lib.get_full_ca_file_path")
    def test_build_as_df_build_args_success(
        self, mock_ca_path, mock_podman, mock_get_uid, mock_build, temp_dir
    ):
        """Test build_as_df_build_args when build succeeds."""
        (temp_dir / "package.json").write_text('{"name": "test"}')
        mock_ca_path.return_value = Path("/nonexistent")
        mock_podman.return_value = ""
        mock_get_uid.return_value = "1001"
        mock_build.return_value = True
        self.lib.app_image_id = "sha256:abc123"

        with patch.object(self.lib, "random_string", return_value="testid123"):
            result = self.lib.build_as_df_build_args(
                app_path=temp_dir,
                src_image="quay.io/fedora/nodejs-16",
                dst_image="test-app:latest",
                build_args="",
                s2i_args="",
            )

        assert result is not None
        assert isinstance(result, ContainerTestLib)
        assert result.image_name == "test-app:latest"
        assert result.s2i_image is True
        mock_build.assert_called_once()

    @patch("container_ci_suite.container_lib.ContainerTestLib.get_uid_from_image")
    @patch("container_ci_suite.container_lib.PodmanCLIWrapper.call_podman_command")
    def test_build_as_df_build_args_source_path_not_exists(
        self, mock_podman, mock_get_uid
    ):
        """Test build_as_df_build_args when app_path does not exist."""
        mock_podman.return_value = ""

        result = self.lib.build_as_df_build_args(
            app_path=Path("/nonexistent/app/path"),
            src_image="quay.io/fedora/nodejs-16",
            dst_image="test-app:latest",
        )

        assert result is False

    @patch("container_ci_suite.container_lib.ContainerTestLib.build_image_and_parse_id")
    @patch("container_ci_suite.container_lib.ContainerTestLib.get_uid_from_image")
    @patch("container_ci_suite.container_lib.PodmanCLIWrapper.call_podman_command")
    @patch("container_ci_suite.container_lib.get_full_ca_file_path")
    def test_build_as_df_build_args_get_uid_fails(
        self, mock_ca_path, mock_podman, mock_get_uid, mock_build, temp_dir
    ):
        """Test build_as_df_build_args when get_uid_from_image returns None."""
        (temp_dir / "file.txt").write_text("content")
        mock_ca_path.return_value = Path("/nonexistent")
        mock_podman.return_value = ""
        mock_get_uid.return_value = None

        result = self.lib.build_as_df_build_args(
            app_path=temp_dir,
            src_image="quay.io/fedora/nodejs-16",
            dst_image="test-app:latest",
        )

        assert result is False
        mock_build.assert_not_called()

    @patch("container_ci_suite.container_lib.ContainerTestLib.build_image_and_parse_id")
    @patch("container_ci_suite.container_lib.ContainerTestLib.get_uid_from_image")
    @patch("container_ci_suite.container_lib.PodmanCLIWrapper.call_podman_command")
    @patch("container_ci_suite.container_lib.get_full_ca_file_path")
    def test_build_as_df_build_args_build_fails(
        self, mock_ca_path, mock_podman, mock_get_uid, mock_build, temp_dir
    ):
        """Test build_as_df_build_args when build_image_and_parse_id returns False."""
        (temp_dir / "file.txt").write_text("content")
        mock_ca_path.return_value = Path("/nonexistent")
        mock_podman.return_value = ""
        mock_get_uid.return_value = "1001"
        mock_build.return_value = False

        result = self.lib.build_as_df_build_args(
            app_path=temp_dir,
            src_image="quay.io/fedora/nodejs-16",
            dst_image="test-app:latest",
        )

        assert result is None

    @patch("container_ci_suite.container_lib.ContainerTestLib.build_image_and_parse_id")
    @patch("container_ci_suite.container_lib.ContainerTestLib.get_uid_from_image")
    @patch("container_ci_suite.container_lib.PodmanCLIWrapper.call_podman_command")
    @patch("container_ci_suite.container_lib.get_full_ca_file_path")
    def test_build_as_df_build_args_with_build_args(
        self, mock_ca_path, mock_podman, mock_get_uid, mock_build, temp_dir
    ):
        """Test build_as_df_build_args with build_args parameter."""
        (temp_dir / "package.json").write_text('{"name": "test"}')
        mock_ca_path.return_value = Path("/nonexistent")
        mock_podman.return_value = ""
        mock_get_uid.return_value = "1001"
        mock_build.return_value = True
        self.lib.app_image_id = "sha256:abc123"

        with patch.object(self.lib, "random_string", return_value="testid456"):
            result = self.lib.build_as_df_build_args(
                app_path=temp_dir,
                src_image="quay.io/fedora/nodejs-16",
                dst_image="test-app:latest",
                build_args="--build-arg NODE_ENV=production",
                s2i_args="",
            )

        assert result is not None
        call_args = mock_build.call_args
        assert "NODE_ENV=production" in str(call_args)

    @patch("container_ci_suite.container_lib.ContainerTestLib.get_uid_from_image")
    @patch("container_ci_suite.container_lib.PodmanCLIWrapper.call_podman_command")
    def test_build_as_df_build_args_pull_fails(
        self, mock_podman, mock_get_uid, temp_dir
    ):
        """Test build_as_df_build_args when image pull fails."""
        (temp_dir / "file.txt").write_text("content")
        mock_podman.side_effect = [
            CalledProcessError(1, "podman images", "not found"),
            CalledProcessError(1, "podman pull", "pull failed"),
        ]

        result = self.lib.build_as_df_build_args(
            app_path=temp_dir,
            src_image="quay.io/fedora/nodejs-16",
            dst_image="test-app:latest",
        )

        assert result is False

    @patch("container_ci_suite.container_lib.ContainerTestLib.build_image_and_parse_id")
    @patch("container_ci_suite.container_lib.ContainerTestLib.get_uid_from_image")
    @patch("container_ci_suite.container_lib.PodmanCLIWrapper.call_podman_command")
    @patch("container_ci_suite.container_lib.get_full_ca_file_path")
    def test_build_as_df_build_args_with_s2i_env_args(
        self, mock_ca_path, mock_podman, mock_get_uid, mock_build, temp_dir
    ):
        """Test build_as_df_build_args with s2i_args containing env vars."""
        (temp_dir / "app.js").write_text("console.log('hello')")
        mock_ca_path.return_value = Path("/nonexistent")
        mock_podman.return_value = ""
        mock_get_uid.return_value = "1001"
        mock_build.return_value = True
        self.lib.app_image_id = "sha256:abc123"

        with patch.object(self.lib, "random_string", return_value="testid789"):
            result = self.lib.build_as_df_build_args(
                app_path=temp_dir,
                src_image="quay.io/fedora/nodejs-16",
                dst_image="test-app:latest",
                build_args="",
                s2i_args="-e NODE_ENV=production",
            )

        assert result is not None

    @patch("container_ci_suite.container_lib.ContainerTestLib.build_as_df_build_args")
    def test_build_as_df_delegates_to_build_args(self, mock_build_args, temp_dir):
        """Test build_as_df delegates to build_as_df_build_args with empty build_args."""
        (temp_dir / "app.js").write_text("")
        mock_build_args.return_value = ContainerTestLib("test:latest", s2i_image=True)

        result = self.lib.build_as_df(
            app_path=temp_dir,
            src_image="quay.io/fedora/nodejs-16",
            dst_image="test-app:latest",
            s2i_args="-e FOO=bar",
        )

        assert result is not None
        mock_build_args.assert_called_once_with(
            temp_dir, "quay.io/fedora/nodejs-16", "test-app:latest", "", "-e FOO=bar"
        )


class TestMultistageBuild:
    """Test multistage_build function."""

    def setup_method(self):
        self.lib = ContainerTestLib()

    @patch("container_ci_suite.container_lib.ContainerTestLib.build_image_and_parse_id")
    @patch("container_ci_suite.container_lib.ContainerTestLib.get_uid_from_image")
    @patch("container_ci_suite.container_lib.PodmanCLIWrapper.call_podman_command")
    @patch("container_ci_suite.container_lib.get_full_ca_file_path")
    def test_multistage_build_success(
        self, mock_ca_path, mock_podman, mock_get_uid, mock_build, temp_dir
    ):
        """Test multistage_build when build succeeds."""
        (temp_dir / "app.js").write_text("console.log('hello')")
        mock_ca_path.return_value = Path("/nonexistent")
        mock_podman.return_value = "1001"
        mock_get_uid.return_value = "1001"
        mock_build.return_value = True
        self.lib.app_image_id = "sha256:abc123"

        result = self.lib.multistage_build(
            app_path=temp_dir,
            src_image="quay.io/fedora/nodejs-20",
            sec_image="quay.io/fedora/nodejs-20-minimal",
            dst_image="test-app:latest",
            s2i_args="",
        )

        assert result is True
        mock_build.assert_called_once()

    @patch("container_ci_suite.container_lib.ContainerTestLib.get_uid_from_image")
    @patch("container_ci_suite.container_lib.PodmanCLIWrapper.call_podman_command")
    def test_multistage_build_get_uid_returns_none(
        self, mock_podman, mock_get_uid, temp_dir
    ):
        """Test multistage_build when get_uid_from_image returns None."""
        (temp_dir / "app.js").write_text("content")
        mock_podman.return_value = "1001"
        mock_get_uid.return_value = None

        result = self.lib.multistage_build(
            app_path=temp_dir,
            src_image="quay.io/fedora/nodejs-20",
            sec_image="quay.io/fedora/nodejs-20-minimal",
            dst_image="test-app:latest",
        )

        assert result is False

    @patch("container_ci_suite.container_lib.ContainerTestLib.build_image_and_parse_id")
    @patch("container_ci_suite.container_lib.ContainerTestLib.get_uid_from_image")
    @patch("container_ci_suite.container_lib.PodmanCLIWrapper.call_podman_command")
    @patch("container_ci_suite.container_lib.get_full_ca_file_path")
    def test_multistage_build_fails(
        self, mock_ca_path, mock_podman, mock_get_uid, mock_build, temp_dir
    ):
        """Test multistage_build when build_image_and_parse_id returns False."""
        (temp_dir / "app.js").write_text("content")
        mock_ca_path.return_value = Path("/nonexistent")
        mock_podman.return_value = "1001"
        mock_get_uid.return_value = "1001"
        mock_build.return_value = False

        result = self.lib.multistage_build(
            app_path=temp_dir,
            src_image="quay.io/fedora/nodejs-20",
            sec_image="quay.io/fedora/nodejs-20-minimal",
            dst_image="test-app:latest",
        )

        assert result is False

    @patch("container_ci_suite.container_lib.ContainerTestLib.build_image_and_parse_id")
    @patch("container_ci_suite.container_lib.ContainerTestLib.get_uid_from_image")
    @patch("container_ci_suite.container_lib.PodmanCLIWrapper.call_podman_command")
    @patch("container_ci_suite.container_lib.get_full_ca_file_path")
    def test_multistage_build_podman_inspect_fails_uses_default_user(
        self, mock_ca_path, mock_podman, mock_get_uid, mock_build, temp_dir
    ):
        """Test multistage_build uses user 0 when podman inspect raises."""
        (temp_dir / "app.js").write_text("content")
        mock_ca_path.return_value = Path("/nonexistent")
        mock_podman.side_effect = CalledProcessError(1, "podman", "inspect failed")
        mock_get_uid.return_value = "0"
        mock_build.return_value = True
        self.lib.app_image_id = "sha256:abc123"

        result = self.lib.multistage_build(
            app_path=temp_dir,
            src_image="quay.io/fedora/nodejs-20",
            sec_image="quay.io/fedora/nodejs-20-minimal",
            dst_image="test-app:latest",
        )

        assert result is True
        mock_get_uid.assert_called_once_with("0", "quay.io/fedora/nodejs-20")

    @patch("container_ci_suite.container_lib.utils.clone_git_repository")
    @patch("container_ci_suite.container_lib.ContainerTestLib.build_image_and_parse_id")
    @patch("container_ci_suite.container_lib.ContainerTestLib.get_uid_from_image")
    @patch("container_ci_suite.container_lib.PodmanCLIWrapper.call_podman_command")
    @patch("container_ci_suite.container_lib.get_full_ca_file_path")
    def test_multistage_build_git_clone_fails(
        self, mock_ca_path, mock_podman, mock_get_uid, mock_build, mock_clone, temp_dir
    ):
        """Test multistage_build when app_path does not exist and git clone fails."""
        mock_ca_path.return_value = Path("/nonexistent")
        mock_podman.return_value = "1001"
        mock_get_uid.return_value = "1001"
        mock_clone.return_value = False

        result = self.lib.multistage_build(
            app_path=Path("/nonexistent/git/repo"),
            src_image="quay.io/fedora/nodejs-20",
            sec_image="quay.io/fedora/nodejs-20-minimal",
            dst_image="test-app:latest",
        )

        assert result is False
        mock_build.assert_not_called()

    @patch("container_ci_suite.container_lib.ContainerTestLib.build_image_and_parse_id")
    @patch("container_ci_suite.container_lib.ContainerTestLib.get_uid_from_image")
    @patch("container_ci_suite.container_lib.PodmanCLIWrapper.call_podman_command")
    @patch("container_ci_suite.container_lib.get_full_ca_file_path")
    def test_multistage_build_with_s2i_args(
        self, mock_ca_path, mock_podman, mock_get_uid, mock_build, temp_dir
    ):
        """Test multistage_build with s2i_args parameter."""
        (temp_dir / "package.json").write_text('{"name": "test"}')
        mock_ca_path.return_value = Path("/nonexistent")
        mock_podman.return_value = "1001"
        mock_get_uid.return_value = "1001"
        mock_build.return_value = True
        self.lib.app_image_id = "sha256:abc123"

        result = self.lib.multistage_build(
            app_path=temp_dir,
            src_image="quay.io/fedora/nodejs-20",
            sec_image="quay.io/fedora/nodejs-20-minimal",
            dst_image="test-app:latest",
            s2i_args="-e NODE_ENV=production",
        )

        assert result is True
        mock_build.assert_called_once()
